#!/usr/bin/perl -w
#
#  Data integrity check tool for cdDataGen
#
#  -------------------------------------------------------------
#  Author: Gerald Klinkl, SIS PRO HPS, gerald.klinkl@siemens.com
#  Last modification $Date: 2009-08-31 11:45:34 $
#  $Revision: 1.1 $
#  -------------------------------------------------------------
#  Perl script for checking waveform files created from data
#  generated with cdDataGen
#
#  Usage: cdCheck.pl waveform_file [waveform_file ...]
#  -------------------------------------------------------------
#  ToDo:
#  -------------------------------------------------------------
#
#  $Log: cdCheck.pl,v $
#  Revision 1.1  2009-08-31 11:45:34  gerald
#  Add utility script which checks data generated by
#  cdDataGen
#

my $DURATION = 10;
my $SAMPRATE = 40;
my $NSAMP = $DURATION * $SAMPRATE;
my $ONE_DAY = 86400;

my ($siteName, $channelName, $channelOffset, $expSampleRate, $dataType);
my $debugLevel = 1;

unless (@ARGV > 0)
       { die "usage: $0 waveform_file [waveform_file ...]\n"; }

###########################################################
#  Debug function
###########################################################

sub debug()
{
   print "DEBUG:$_[0]\n" if ($debugLevel);
}

###########################################################
# Check data
###########################################################

while (@ARGV)
{
   my $actWF = shift(@ARGV);
   my $ii;
   my $sampLng;

   open(MWF,"<$actWF") or die "can't open $actWF";
   # read header lines
   my %channels = ();
   for ($ii = 0; $ii < 100; $ii++)
   {
      $_ = <MWF>;
      next unless defined($_);
      ($siteName, $channelName, $channelOffset, $expSampleRate, $dataType) =
            split / +/;
      next unless defined($dataType);
      chop $dataType;
      $sampLng = $dataType;
      &debug("[$siteName/$channelName], Offset [$channelOffset], type [$dataType]");
      $channels{"$siteName/$channelName"} = $channelOffset;
   }
   close WMF;
   &debug("NSAMP = $NSAMP, sampLng = $sampLng");
   foreach $channel (reverse keys %channels)
   {
      &debug("[$channel], Offset " . $channels{$channel});
      seek MWF, $channels{$channel}, 0 or die "can't seek";
      my $channelID = 0;
      my $lastValue = -1;
      for ($ii = 0; $ii < $ONE_DAY/$DURATION; $ii++)
      {
         # read channel ID 
         read MWF, $input, $sampLng, 1 or 
              die "canot read at offset $channels{$channel}";
         ($value) = unpack "N", $input;
         $value &= 0x00FFFFFF;
         if ($channelID == 0)
         {
            &debug("channelID = $value");
            $channelID = $value;
         }
         elsif ($channelID != $value )
         {
            die "channel ID check failed\n\tfile $actWF\n\tchannel $channel" .
                 "\n\tsample $ii/$ij\n\toffset " . tell MWF;
         }
         # read channel start time
         read MWF, $input, $sampLng, 1 or 
              die "canot read at offset $channels{$channel}";
         my ($actTime) = unpack "N", $input;
         $actTime &= 0x0000FFFF;
         read MWF, $input, $sampLng, 1 or 
              die "canot read at offset $channels{$channel}";
         ($value) = unpack "N", $input;
         $value &= 0x00FFFFFF;
         for ($ij = 3; $ij < $NSAMP; $ij++)
         {
            read MWF, $input, $sampLng, 1 or 
                 die "canot read at offset $channels{$channel}";
            ($value) = unpack "N", $input;
            $value &= 0x00FFFFFF;
# &debug("value = [$value]");
            $lastValue++;
            if ($value != $lastValue)
            {
               die "value check failed, file $actWF, channel $channel, sample $ii/$ij";
            }
         }
      }
   }
   print "$actWF: test passed\n";
}
