#!/usr/bin/perl -w

##search for WORK_QUERY to jum directly to query state function

#
#  GTK UI for CD controller
#
#  ----------------------------------------------------------
#  Author: Gerald Klinkl, PSE KB22, gerald.klinkl@siemens.com
#  Last modification $Date: 2007-09-13 16:08:40 $
#  $Revision: 1.12 $
#  ----------------------------------------------------------
#  This script implements a Graphical User Interface for
#  CD Controller. It is able to deal with different instances
#  of CD Receiver and CD Sender
#
#  Usage: congui.pl congui_config_file
#  ----------------------------------------------------------
#  ToDo:
#  ----------------------------------------------------------
#
#  $Log: congui.pl,v $
#  Revision 1.12  2007-09-13 16:08:40  gerald
#  Upgrade to perl-gtk2 and mark stations which sends only
#  command request frames specially
#
#  Revision 1.10  2006/10/04 13:04:28  gerald
#  Remove sender specific values from (generic) configuration window
#
#  Revision 1.9  2006/01/31 16:29:45  gerald
#  Update comments
#
#  Revision 1.48  2005/11/14 08:56:52  gerald
#  Add test case for CD Senders filter change command
#

# ===============================================
# Documentation for congui.pl configuration file:
#
# The congui config file stores information in the following way:
# A '#' at the start or a line is used to mark a comment
# </path/to/configurationfile> stationname:ip:port
# You may specify as many configurations as you want.

use Gtk2 '-init';
use Glib qw(TRUE FALSE);
use strict;
use Data::Dumper;

set_locale Gtk2;
init Gtk2;

if (@ARGV !=1)
{ die "usage: $0 config_file\n";}

#used to determine wether a station has to be deleted from the tree or not
my $alive = 1;
my $dead = 0;
my %station_state;

use constant {
   NOTEBOOK => 0,
   ICON => 1,
   TREE => 2
};

my $outView = TREE;


my $debug = 1;
my $CDcontrol = "../src/cdcon/cdcon";
my $configFile = shift(@ARGV);
# appsList will hold the CDapps specified in configFile,the actual
# subtree-node (iter) and row information for debugging
my @appsList;
my @stationNodes;
my @stationArray;
my $ii = 0;
# parse congui config file and extract infos
my $jj = 0; # appsList counter

open(CFG, "<$configFile") or die "cannot open $configFile\n";

while(<CFG>)
{
   # cfgList identifies the CDapp in stationArray
   my ($cfgFile, $cfgList);

   if (/^#/) { next; } # Comment line
   $cfgList = '';
   ($cfgFile, $cfgList) = split(' ', $_, 2);

   if ($cfgList !~ /.+:.*:\d+/)
   {
      die "\n Wrong input format [$_]\n";
   }

   $appsList[$jj][0] = undef; # initialization for tree node
   $appsList[$jj][1] = $cfgFile;
   $appsList[$jj][2] = undef; # only for debugging (row will be stored in here)
   chop($cfgList);
   $appsList[$jj][3] = $cfgList;
   
   $jj++;
} # end while(<CFG>)
close(CFG);

# because value of $jj is off-by-one. And $jj is used to show the
# length of appsList further down. (query_state())
$jj--;

use constant {
    ST_NAME  => 0,
    NODE_TYPE => 1,
    CFG_INDEX => 2,
    ST_TYPE  => 3,
    RCV_BYTES  => 4,
    SND_BYTES  => 5,
    DBG_TXT  => 6,
    ST_ICON  => 7 
};

use constant {
   RECEIVER => 0,
   SENDER => 1,
   ROOT => 2,
   STATION => 3
};

my @states = ('int. error', 'unconnected', 'connecting',
              'waiting', 'connected', 'wait2send', 'stopped');
my @types = ('int. error', 'N.A.', 'CD1.0', 'CD1.1');
my @a_states = ('N.A.', 'no keys', 'off', 'active');
my $station = '';

my $false = 0;
my $true = 1;

my $isNotInit = 1;

my $passWin;
my $window;
my $scrolled_window;
my $model;
my $node;
my $root;

my ( @station_red_xpm, @station_yellow_xpm, @station_red_green_xpm,
     @station_green_xpm, @station_black_xpm, @station_wait_xpm,
     @station_green_red_xpm, @station_green_yellow_xpm, 
     @receiver_xpm, @sender_xpm, @root_xpm, @controller_xpm);

my ($systemConf, $debugLevel, $oldDebugLevel, $oldSystemConf);

my ($val0 ,$val1 ,$val2 ,$val3 ,$val4 ,$val5 ,$val6 ,$val7 ,$val8 ,$val9 ,
    $val10 ,$val11 ,$val12 ,$val13);

my %cfgHash;
my $errWin = undef;
#error window, open via openERR(<errorMsg>,<additionalInfo>);


# Create Window
# EXPL: dialog instead of window because it comes with a vbox and 
# hbox(calles action area) already inserted
$window = new Gtk2::Dialog();
$window->set_default_size( 500, 600 );
$window->set_title( "CD Controller GUI V0.2" );
$window->signal_connect('destroy' => sub { Gtk2->main_quit; });
$window->set_border_width( 5 );
$window->realize();

# Set up Pixmaps
&setupPixmaps;

sub loadPixbuf()
{
   return(Gtk2::Gdk::Pixbuf->new_from_xpm_data(@_));
}

my $root_icon = &loadPixbuf(@root_xpm);
my $receiver_icon = &loadPixbuf(@receiver_xpm);
my $sender_icon = &loadPixbuf(@sender_xpm);
my $controller_icon = &loadPixbuf(@controller_xpm);
my $unconnected_icon = &loadPixbuf(@station_red_xpm);
my $error_icon = &loadPixbuf(@station_green_red_xpm);
my $warning_icon = &loadPixbuf(@station_green_yellow_xpm);
my $connecting_icon = &loadPixbuf(@station_red_green_xpm);
my $good_icon = &loadPixbuf(@station_green_xpm);
my $stopped_icon = &loadPixbuf(@station_yellow_xpm);
my $wait_icon = &loadPixbuf(@station_wait_xpm);
my $unknown_icon = &loadPixbuf(@station_black_xpm);

# Create a handlebox for a toolbar and fill the toolbar with a station legend
#
my $handlebox = new Gtk2::HandleBox();
$handlebox->set_shadow_type( 'out' );
$handlebox->set_handle_position( 'left' );
$handlebox->set_snap_edge( 'left' );
$window->action_area->set_homogeneous( $true );
$window->action_area->pack_start( $handlebox, $true, $false, 0);

# Create the toolbar
my $toolbar = new Gtk2::Toolbar();
$toolbar->set_orientation('horizontal');
$toolbar->set_style('both');
$toolbar->set_border_width( 11 );
## $toolbar->set_space_size( 5 );
$handlebox->add( $toolbar );

# Create vbox for two tables *grrr*
my $toolbar_vbox = new Gtk2::VBox( $false, 1 );
$toolbar->append_widget($toolbar_vbox,"","");

# Pack a table into toolbar to get a nicer look for the stationlegend
# upper table
my $toolbar_table_upper = new Gtk2::Table( 2, 10, $false );
$toolbar_vbox->pack_start_defaults( $toolbar_table_upper);
$toolbar_table_upper->set_row_spacings( 1 );

# horizontal separator added to get a better overview of the available 
# station states
my $hbar = new Gtk2::HSeparator();
$toolbar_table_upper->attach_defaults( $hbar,0,10,1,2);
$hbar->show();

# lower table
my $toolbar_table_lower = new Gtk2::Table( 1, 10, $false );
$toolbar_vbox->pack_start_defaults( $toolbar_table_lower);
$toolbar_table_lower->set_row_spacings( 1 );

use constant {
   INTERNAL_FAILURE    => 0,
   STATION_UNCONNECTED => 1,
   STATION_CONNECTING  => 2,
   STATION_WAITING     => 3,
   STATION_CONNECTED   => 4,
   STATION_WAIT2SEND   => 5,
   STATION_STOPPED     => 6,
   # next ones are only for congui
   STATION_HAS_WARNINGS => 7,
   STATION_HAS_ERRORS   => 8,
   STATION_IS_CONTROL   => 9
};
   
# all stationpixmaps, descriptions and object references for the table are 
# stored here. They appear in the legend in the same order as in this array
my @station_codes = ([$unknown_icon,"Unknown\nstate",undef],
                     [$unconnected_icon,"Station is\nunconnected",undef],
                     [$connecting_icon,"Station is\nconnecting",undef],
                     [$connecting_icon,"Station is\nwaiting",undef],
                     [$good_icon,"Station is\nconnected",undef],
                     [$wait_icon,"Station has\ndata to send",undef],
                     [$stopped_icon,"Station has\nbeen stopped",undef],
                     [$warning_icon,"Station has\nwarnings",undef],
                     [$error_icon,"Station has\nerrors",undef],
                     [$controller_icon,"Controller\nStation",undef]);

my ($ff, $pos, $icon_image);

# fill in information from @station_codes into the toolbar
for ($ff = 0, $pos = 0; $ff < (scalar @station_codes); $ff++, $pos = $pos + 2)
{
   if ($pos >= 10) { $pos = 0; }

   # switch to 2nd row after 5 elements
   if ($ff < 5) 
   {
     $toolbar_table_upper->set_col_spacing($pos, 5);
     $toolbar_table_upper->set_col_spacing($pos + 1, 10);

     # add station symbol to toolbar
     $icon_image = Gtk2::Image->new_from_pixbuf($station_codes[$ff][0]);
     $toolbar_table_upper->attach_defaults($icon_image, $pos,
                                           $pos + 1, 0, 1);
     # show is neccessary because widget is not created inside toolbar    
     $icon_image->show();

     # add station symbols description to toolbar
     $station_codes[$ff][2] = new Gtk2::Label($station_codes[$ff][1]);
     $toolbar_table_upper->attach_defaults($station_codes[$ff][2], $pos + 1,
                                           $pos + 2, 0, 1);
     $station_codes[$ff][2]->show();
   } 
   else 
   {
     $toolbar_table_lower->set_col_spacing($pos, 5);
     $toolbar_table_lower->set_col_spacing($pos + 1, 10);
     
     # add station symbol to toolbar
     $icon_image = Gtk2::Image->new_from_pixbuf($station_codes[$ff][0]);
     $toolbar_table_lower->attach_defaults($icon_image, $pos,
                                           $pos + 1, 0, 1);
     #show is neccessary because widget is not created inside toolbar    
     $icon_image->show();
     
     # add station symbols description to toolbar
     $station_codes[$ff][2] = new Gtk2::Label($station_codes[$ff][1]);
     $toolbar_table_lower->attach_defaults($station_codes[$ff][2],$pos + 1,
                                           $pos + 2, 0, 1);
     $station_codes[$ff][2]->show();
   }
}# end for

$toolbar_vbox->show();
$toolbar_table_upper->show();
$toolbar_table_lower->show();
$toolbar->show();
$handlebox->show();


# Get window style and colormap
my $style = new Gtk2::Style;
my $cm = $window->get_colormap;

sub getColor()
{
   my $color = Gtk2::Gdk::Color->new(@_);
   $cm->alloc_color($color,TRUE,TRUE);
   return($color);
}

my $greybg = &getColor(60000, 60000, 60000);
my $white = &getColor(65000, 65000, 65000);
my $blue = &getColor(0, 0, 65000);
my $col_error = &getColor(65000, 24735, 12495);
my $col_warning = &getColor(55000, 38450, 19325);
my $greyInactive = &getColor(40000, 40000, 40000);
my $black = &getColor(0, 0, 0);
my $ctbto_light_blue = &getColor(56832, 60160, 57856);
my $ctbto_blue = &getColor(48640, 55040, 54784);
my $ctbto_dark_blue = &getColor(42032, 49920, 49664);
my $stylebg = $window->get_style()->bg( 'normal' );

my $titleBoxCol =  $ctbto_dark_blue;
my $titleBoxFg = $black;
my $windowCol = $ctbto_light_blue;
# font

# Create Scrolled Window
$scrolled_window = new Gtk2::ScrolledWindow( undef, undef );
$window->vbox->pack_start( $scrolled_window,$true,$true,0 );
$scrolled_window->set_policy( 'automatic', 'always' );
# $scrolled_window->show();


# Create CTree
my $timer = 0;
query_state();
# called periodically, every 10 seconds
$timer = Glib::Timeout->add( 10000, \&query_state );
# $tree->show();
$window->show_all();

Gtk2->main;
exit 0;



### Subroutines


sub debug
{
   print "DEBUG: @_" if $debug;
}

# Given the tree, parent node (undef for root node), text, and type
# of the node, add the node to the tree.

sub insert_node
{
   my ($model, $parent, $text, $icon, $node_type, $appsListIndex) = @_;
   my $iter = $model->append($parent);

   $model->set($iter, ST_ICON, $icon,
                      ST_NAME, @$text[0],
                      ST_TYPE, @$text[1],
                      RCV_BYTES, @$text[2],
                      SND_BYTES, @$text[3],
                      DBG_TXT, @$text[4],
                      NODE_TYPE, $node_type,
                      CFG_INDEX, $appsListIndex);
   
   return($iter);
}

my (%text_fields);



# Add field with label
#
sub addTextField()
{
   my ($text, $fieldText, $table, $column, $row, $color) = @_;
   my $bgColor = $ctbto_light_blue;

   my ($alignment, $label) = &getField($text, $black, $bgColor, 1);
   $table->attach_defaults($alignment, $column, $column + 1, $row, $row + 1);

   if (defined($color) && $fieldText > 0) { $bgColor = $color; }

   $column++;
   ($alignment, $label) = &getField($fieldText, $blue, $bgColor, 0);
   $table->attach_defaults($alignment, $column, $column + 1, $row, $row + 1);
   $text_fields{$text} = $label;
}

# Add a right adjusted table entry
#
sub addTableEntry()
{

   my $label = new Gtk2::Label(shift(@_));
   my $table = shift(@_);
   my $hbox = new Gtk2::HBox( $false, 0 );
   # use hbox for justification
   $hbox->pack_end( $label, $false, $false, 0 );
   $table->attach_defaults($hbox, @_);
}

#not in use
# Add a left adjusted table entry
#
#sub addTableEntryLeft()
#{
#   my $label = new Gtk2::Label(shift(@_));
#   my $table = shift(@_);
#   my $hbox = new Gtk2::HBox( $false, 0 );
#   my $style2 = $label->style;
#   $style2 = $style2->copy;
#   $style2->fg('normal', $blue);
#
#   $hbox->pack_start( $label, $false, $false, 0 );
#   $table->attach_defaults($hbox, @_);
#   $label->set_style($style2);
#}

sub updateDetails()
{
   my ($widget, $par) = @_;
   my ($response, $ok, $field, $value);

   print Dumper(@_);

   my $cmd = "$CDcontrol $appsList[@$par[1]][1] $appsList[@$par[1]][3] " . 
             "GST=4,@$par[0]";

   debug "$cmd\n";
   open (CMD, "$cmd |") or die "can't execute $cmd";
   $ok = 'no';

   # read station information
   while (<CMD>)
   {
      if ($ok eq 'yes')
      {
         chop($_);

         $response = $_;
      }
      else
      {
         debug "Read: $_";
      }
      if ($_ =~ /^000 OK/) { $ok = 'yes'; }
   }
   close(CMD);

   for $field ('', 'State:', 'Type:', 'Port:', 'Auth.state:',
               '', 'CR warnings:', 'CR errors:', 'Warnings:', 'Errors:',
               '  Frames received:', '  Frames sent:', '  Bytes received:',
               '  Bytes sent:', '', '', '', '',
               'is connected:', 'invalid station:', 'invalid IP:',
               'authentication:', 'invalid frame:', 'timeout:',
               'unexpected close:', 'system failure:',
               '', ' invalid station:', ' invalid IP:',
               ' authentication:', ' invalid frame:', ' timeout:',
               ' unexpected close:', ' system failure:')
   {
      ($value, $response) = split(' ', $response, 2);
      debug "field [$field] value [$value]\n";

      if ('' ne $field)
      {
         if ('State:' eq $field) { $value = $states[$value]; }
         elsif ('Type:' eq $field) { $value = $types[$value]; }
         elsif ('Auth.state:' eq $field) { $value = $a_states[$value]; }

         $text_fields{$field}->set_text($value);
      }
   }


#   my ($nu, $state, $type, $port, $a_state, $crs, $cr_warn,
#       $cr_err, $warn, $err, $framesRec, $framesSent, $bytesRec,
#       $bytesSent, $nu, $nu, %nu, $nu, $isConCR, $unAuthStationCR, $unAuthIPCR,
#       $authFailCR, $invFramesCR, $timeoutCR, $unexCloseCR, $sysFailCR,
#       isCon, $unAuthStation, $unAuthIP,
#       $$authFail, $invFrames, $timeout, $unexClose, $sysFail,
#       ) = split(/ /, $response, 32);


}

sub getField()
{
   my ($text, $fgColor, $bgColor, $align) = @_;


   my $alignment = new Gtk2::Alignment( $align, 0.5, 0, 0 );
   # A label has no background, so we use an event box
   my $label = new Gtk2::Label( $text);
   $label->modify_fg('normal', $fgColor );
   my $vbox = new Gtk2::VBox( $false, 5 );
# $vbox->pack_start( $alignment, $false, $false, 0 );
   my $event_box = new Gtk2::EventBox();
   $event_box->modify_bg('normal', $bgColor );
   $event_box->add( $label )      ;
   $alignment->add( $event_box );
   return($alignment, $label);
}

my $detailWin = undef;
my $popUp = undef;


# Create a detail window  window statistic data
#
sub openDetail
{
   my ($widget, $event, $par) = @_;

   $popUp->destroy() unless (! defined($popUp));
   $detailWin->destroy() unless (! defined($detailWin));
   $detailWin = new Gtk2::Window( "toplevel" );

   $detailWin->modify_bg('normal', $windowCol);
   $detailWin->realize();
   $detailWin->set_decorated(TRUE);
   $detailWin->set_resizable( TRUE );
   $detailWin->set_position('mouse' );
   $detailWin->set_title( "Station Details" );
   $detailWin->signal_connect( "destroy", sub { $detailWin->destroy(); } );
   $detailWin->set_border_width( 10 );

   my $frameDetail = new Gtk2::Frame();
   $frameDetail->set_shadow_type( 'none' );
   $detailWin->add($frameDetail);

   # Add vbox for the two tables
   my $vboxDetail = new Gtk2::VBox($false, 10 );
   $frameDetail->add( $vboxDetail );

   # table
   my $tableDetail = new Gtk2::Table( 10, 10, $false );
   $tableDetail->set_row_spacings( 5 );
   $tableDetail->set_col_spacings( 5 );
   $vboxDetail->pack_start( $tableDetail, $false, $false, 0 );

   # row1, empty
   # row2
   # a label has no background, so an event_box is used
   my ($alignment, $not_used) = &getField(@$par[0], $titleBoxFg,
                                          $titleBoxCol, 1);
   $tableDetail->attach_defaults($alignment, 4, 6, 1, 2);

   # row3 empty (2,3)

   debug $stationArray[@$par[1]]{@$par[0]} . "\n";
   my ($nu, $state, $type, $port, $a_state, $crs, $cr_warn,
       $cr_err, $warn, $err, $framesRec, $framesSent, $bytesRec,
       $bytesSent) = split(" ", $stationArray[@$par[1]]{@$par[0]});

   # row4
   &addTableEntry('    ', $tableDetail, 0, 1, 3, 4);
   &addTextField('State:', $states[$state], $tableDetail, 1, 3);
   &addTextField('Type:', $types[$type], $tableDetail, 3, 3);
   &addTextField('Port:', $port, $tableDetail, 5, 3);
   &addTextField('Auth.state:', $a_states[$a_state], $tableDetail, 7, 3);
   &addTableEntry('    ', $tableDetail, 9, 10, 3, 4);

   # row5 empty

   # row6
   &addTextField('  Frames received:', $framesRec, $tableDetail, 1, 4);
   &addTextField('  Frames sent:', $framesSent, $tableDetail, 3, 4);
   &addTextField('  Bytes received:', $bytesRec, $tableDetail, 5, 4);
   &addTextField('  Bytes sent:', $bytesSent, $tableDetail, 7, 4);

   # Create frame for overview
   my $frame0 = new Gtk2::Frame('Total amount of Warnings/Errors ');
   $frame0->set_shadow_type( 'in' );
   $vboxDetail->pack_start( $frame0, $false, $false, 0 );

   #
   my $tableFr0 = new Gtk2::Table( 4 , 9, $false );#11,9,$false
   $tableFr0->set_row_spacings( 5 );
   $tableFr0->set_col_spacings( 5 );
   $frame0->add( $tableFr0 );

   # row 1
   my ($heading1, $labelTabFr0) = &getField('unconnected state', $black,
                                            $ctbto_light_blue, 1);
   $labelTabFr0->set_pattern( '__________________________________________ ');
   $tableFr0->attach_defaults($heading1, 1, 3, 1, 2);

   my ($heading2, $label_1_TabFr0) = &getField('connected state', $black,
                                               $ctbto_light_blue, 1);
   $label_1_TabFr0->set_pattern( '__________________________________________ ');
   $tableFr0->attach_defaults($heading2, 3, 5, 1, 2);

   #row 2 to 4
   &addTextField('CR warnings:', $cr_warn, $tableFr0, 2, 2,$col_warning);
   &addTextField('CR errors:', $cr_err, $tableFr0, 2, 3,$col_error);
   &addTextField('Warnings:', $warn, $tableFr0, 4, 2,$col_warning);
   &addTextField('Errors:', $err, $tableFr0, 4, 3,$col_error);


   my $frame1 = new Gtk2::Frame('Detailed Information');
   $frame1->set_shadow_type( 'in' );

   $vboxDetail->pack_start( $frame1, $false, $false, 1 );

   #table for content
   my $tableFr1 = new Gtk2::Table( 9, 9, $false );#11,9,$false
   $tableFr1->set_row_spacings( 5 );
   $tableFr1->set_col_spacings( 5 );
   $frame1->add( $tableFr1 );

   # rows  2 - 9
   &addTextField('is connected:', 0, $tableFr1, 2, 2);
   &addTextField('timeout:', 0, $tableFr1, 2, 3);
   &addTextField(' timeout:', 0, $tableFr1, 4, 3);
   &addTextField('invalid IP:', 0, $tableFr1, 2, 4);
   &addTextField(' invalid IP:', 0, $tableFr1, 4, 4);
   &addTextField('invalid station:', 0, $tableFr1, 2, 5);
   &addTextField(' invalid station:', 0, $tableFr1, 4, 5);
   &addTextField('invalid frame:', 0, $tableFr1, 2, 6);
   &addTextField(' invalid frame:', 0, $tableFr1, 4, 6);
   &addTextField('authentication:', 0, $tableFr1, 2, 7);
   &addTextField(' authentication:', 0, $tableFr1, 4, 7);
   &addTextField('unexpected close:', 0, $tableFr1, 2, 8);
   &addTextField(' unexpected close:', 0, $tableFr1, 4, 8);
   &addTextField('system failure:', 0, $tableFr1, 2, 9);
   &addTextField(' system failure:', 0, $tableFr1, 4, 9);

   $tableFr1->show_all();
   # button row
   my $hboxButton = new Gtk2::HBox( $true, 12 );
   $hboxButton->modify_bg('normal', $ctbto_blue );
   $hboxButton->set_style($style);

   my $button = new Gtk2::Button( "Close" );
   $button->signal_connect( 'clicked', sub { $detailWin->destroy(); } );
   $hboxButton->pack_start( $button, $true, $true, 0 );
   my $button1 = new Gtk2::Button( "Refresh" );
   $button1->signal_connect( 'clicked', \&updateDetails, $par);
   $hboxButton->pack_start( $button1, $true, $true, 12 );
   $vboxDetail->pack_start( $hboxButton, $false, $false, 0 );

   #show content
   $detailWin->show_all();
   &updateDetails($nu, $par);
}

my $confWin = undef;

#openConfigure Window; allows to change station configuration and debug level
sub openConf
{
   my ($widget, $event, $par) = @_;

   $popUp->destroy() unless (! defined($popUp));
   $confWin->destroy() unless (! defined($confWin));
   $confWin = new Gtk2::Window( "toplevel" );

   $confWin->modify_bg('normal', $windowCol);
   $confWin->set_decorated(TRUE);
   $confWin->set_resizable( TRUE );
   $confWin->realize();
   $confWin->set_position('mouse' );
   $confWin->set_title( "Station Configuration" );
   $confWin->signal_connect( "destroy", sub { $confWin->destroy(); } );
   $confWin->set_border_width( 10 );
   $confWin->set_default_size(700, 500 );

   #create main frame (just for optical reasons)
   my $frameConf = new Gtk2::Frame();
   $frameConf->set_shadow_type( 'none' );
   $frameConf->set_border_width(10);
   $confWin->add($frameConf);


   #create main table:
   my $tableConf = new Gtk2::Table( 4, 1, $false );
   $tableConf->set_row_spacings(5);
   $tableConf->set_col_spacings(0);
   $frameConf->add($tableConf);

   ################################ headline
   #create vbox in row 1(stores title and more)
   my $vboxConf = new Gtk2::VBox( $true, 10 );
   $tableConf->attach_defaults($vboxConf, 0, 1, 0, 1);

   my $headtable = createQuarterTable(10,10,$false);
   $headtable->set_row_spacings( 5 );
   $vboxConf->pack_start_defaults($headtable);

   # row2
   # a label has no background, so an event_box is used
   my ($event_box, $labelHead) = &getField(@$par[0], $titleBoxFg,
                                           $titleBoxCol, 1);
   $headtable->attach_defaults($event_box, 3, 5, 1, 2);

   # row3 empty (2,3)

   my ($nu, $state, $type, $port, $a_state, $crs, $cr_warn,
       $cr_err, $warn, $err, $framesRec, $framesSent, $bytesRec,
       $bytesSent) = split(" ", $stationArray[@$par[1]]{@$par[0]});

   # row4
   &addTableEntry('    ', $headtable, 0, 1, 3, 4);
   &addTextField('State:', $states[$state], $headtable, 1, 3);
   &addTextField('Type:', $types[$type], $headtable, 3, 3);
   &addTextField('Port:', $port, $headtable, 5, 3);
   &addTextField('Auth.state:', $a_states[$a_state], $headtable, 7, 3);
   &addTableEntry('    ', $headtable, 9, 10, 3, 4);

   # row5 empty

   # row6
   &addTextField('  Frames received:', $framesRec, $headtable, 1, 4);
   &addTextField('  Frames sent:', $framesSent, $headtable, 3, 4);
   &addTextField('  Bytes received:', $bytesRec, $headtable, 5, 4);
   &addTextField('  Bytes sent:', $bytesSent, $headtable, 7, 4);
   ########################## end headline

   ###get systemConf values
   $systemConf = 0;
   $debugLevel = 0;

   my $msg;
   my $OK;
   my $cmd;

   $cmd = "$CDcontrol $appsList[@$par[1]][1] $appsList[@$par[1]][3] " . 
          "GCP=@$par[0]";
   open (CMD, "$cmd |") or die "can't execute $cmd)";
   while (<CMD>)
   {
           debug $_;
            ($msg,$OK,$systemConf) = split(" ",$_,3);
   }
   close(CMD);
   chop ($systemConf);
   debug "$systemConf\n";

   $oldSystemConf = $systemConf;

   if(check_perms($systemConf,$cmd) == 0) { return 0;}

   #####get debuglvl
   $cmd = "$CDcontrol $appsList[@$par[1]][1] $appsList[@$par[1]][3] " . 
          "GDL=@$par[0]";
   open (CMD, "$cmd |") or die "can't execute $cmd)";
   while (<CMD>)
   {
      debug $_;
      ($msg,$OK,$debugLevel) = split(" ",$_,3);
   }
   close(CMD);
   chop ($debugLevel);
   
   #reason: createWidget() gets its values from  cfgHash
   $cfgHash{'debugLevel_t'} = $debugLevel;
   #debug "\nDEBUG: $cfgHash{'debugLevel_t'}\n";
   $oldDebugLevel = $debugLevel;

   #check permissions
   if(check_perms($debugLevel,$cmd) == 0) { return 0;}

   &splitSystemconf;

   ##################################
   #row 2 will be used for fileformats and logging

   #col 1
   my $hbox1 = new Gtk2::HBox( $false, 10 );
   $tableConf->attach($hbox1, 0, 1, 1, 2, ['shrink', 'fill'], 'shrink', 5, 2);

   my $frame1 = createQuarterFrame('File Checker');
   $hbox1->pack_start_defaults( $frame1 );

   my $files = createQuarterTable(4,2,$false);
   $frame1->add($files);

   #fileformats
   my $binFiles = createWidget($files,'enable BinFile',0,1,0,1,
                               'checkbutton','enableBinFile_t');

   my $clFiles = createWidget($files,'enable ClfFile',0,1,1,2,
                              'checkbutton','enableClfFile_t');

   my $txtFiles = createWidget($files,'enable TxtFile',0,1,2,3,
                        'optionmenu','enableTxtFile_t',"NO:YES:MORE:FULL");

   my $wfdiscFile = createWidget($files,'enable WfdiscFile',0,1,3,4,
                                 'checkbutton','enableWfdiscFile_t');
   #end col 1

   #col 2
   my $frame2 = createQuarterFrame('Logging');
   $hbox1->pack_start_defaults( $frame2 );

   my $logs = createQuarterTable(5,2,$false);
   $frame2->add($logs);

   #logs
   my $eSyslog = createWidget($logs,'enable Syslog',0,1,0,1,'optionmenu',
                              'enableSyslog_t',"NO:YES:MORE:FULL");

   my $debugLvl = createWidget($logs,'debug Level',0,1,1,2,'textentry',
                               'debugLevel_t');

   my $logConRcs = createWidget($logs,'log ConnectionRequests',0,1,2,3,
                                'optionmenu','logConnectionRequests_t',
                                "NONE:INVALIDONLY:ALL");

   my $absPath =  createWidget($logs,'absWfdiscPath',0,1,3,4,'checkbutton',
                               'absWfdiscPath_t');

   my $logOutg = createWidget($logs,'log Outgoing Data',0,1,4,5,
                              'checkbutton','logOutgoingData_t');
   #end col 2

   ############################ end row2

   ############################ row 3
   #privileges and station modes
   my $hbox2 = new Gtk2::HBox( $false, 10 );
   $tableConf->attach($hbox2, 0, 1, 2, 3, ['shrink', 'fill'], 'shrink', 5, 2);

   my $frame3 = createQuarterFrame('Privileges');
   $hbox2->pack_start_defaults( $frame3 );

   my $priv = createQuarterTable(2,2,$false);
   $frame3->add($priv);

   #privileges
   my $usrPriv = createWidget($priv,'userGroupPrivileges',0,1,0,1,
                              'optionmenu','userGroupPrivileges_t',
                              "NONE:READONLY:READWRITE");

   my $wPriv = createWidget($priv,'worldPrivileges',0,1,1,2,'optionmenu',
                            'worldPrivileges_t',"NONE:READONLY:READWRITE");
   #end col 1

   #col 2
   my $frame4 = createQuarterFrame('Station Modes');
   $hbox2->pack_start_defaults( $frame4 );

   my $modes = createQuarterTable(3,2,$false);
   $frame4->add($modes);

   #station modes
   my $veriSig = createWidget($modes,'veriSigEnable',0,1,0,1,'optionmenu',
                              'verSigEnable_t',"NONE:CHANNEL:FRAME:ALL");

   my $sMode = createWidget($modes,'strictModeEnable',0,1,1,2,'checkbutton',
                            'strictModeEnable_t');

# my $procModes = createWidget($modes,'processMode',0,1,2,3,'optionmenu',
#                                 'processMode_t',"LS_ARRIVED:LIFO:FIFO");
   #end col 2

   ############################ end row 3

   ########################## row 4
   #button row
   my $frame5 = new Gtk2::Frame();
   $frame5->set_shadow_type( 'none' );
   $frame5->set_border_width(10);

   $tableConf->attach($frame5, 0, 1, 3, 4, ['shrink', 'fill'], 'shrink', 0, 0);
   my $button_row = createQuarterTable(1,6,$true);
   $tableConf->set_col_spacings(2);
   $frame5->add($button_row);

   #create set button
   my $button3 = new Gtk2::Button('Set');
   $button3->signal_connect( 'clicked', \&set_values, $par);
   $button_row->attach_defaults($button3,0,1,0,1);

   my $button = new Gtk2::Button( "Close" );
   $button->signal_connect( 'clicked', sub { $confWin->destroy(); } );
   $button_row->attach_defaults($button,5,6,0,1);
   ######################### end row 4

   $confWin->show_all();

}#end openConf

#creates a frame which is used to display the configuration values
sub createQuarterFrame
{
   my($name) = $_;
   my $frame = new Gtk2::Frame($name);

   $frame->set_shadow_type( 'in' );
   $frame->set_border_width(2);

   return $frame;
}

#creates a table which is to be put in the above mentioned frame
sub createQuarterTable
{
   my ($rows,$cols,$homogenous) = @_;

   my $table = new Gtk2::Table($rows,$cols,$false);
#### my $table = new Gtk2::Table($rows,$cols,$homogenous);
   $table->set_row_spacings(2);
   $table->set_col_spacings(5);

   return $table;

}

#creates table entries(checkbuttons,textfields and combos(as option menu)
sub createWidget()
{
   # values should be passed to sub in this order:
   # $table,$name,$x,$y,$top,$bot,$sort,$hash_value,$optmenuopts
   # optmenuopts has to be a doubledot separated list

   #name of configuration value
   my ($alignment, $label) = &getField("$_[1]: ", $black, $ctbto_light_blue, 1);
   $_[0]->attach($alignment,$_[2],$_[3],$_[4],$_[5],['shrink', 'fill'],
                 'shrink', 5, 2);
   my $newWidget;

   #decides which widget has to be created
   if ($_[6] eq 'checkbutton')
   {
      my $val = $cfgHash{$_[7]};
      my $checkbutton = new Gtk2::CheckButton();
      $checkbutton->set_active($val);
      ######debug
      #debug "CHECKBUTTON:  $val \n";
      ######end
      $checkbutton->signal_connect( 'toggled', \&clickB , $_[7], );
      $newWidget = $checkbutton;
   }
   elsif ($_[6] eq 'textentry')
   {
      my $val = $cfgHash{$_[7]};
      my $entry = new Gtk2::Entry();
      $entry->set_max_length(2); 
      $entry->set_visibility($true);
      ####debug
      #debug" TEXTENTRY $val\n";
      ###end
      $entry->set_text($val);
      $entry->signal_connect( 'changed', \&setTextEntry, $_[7] );
      $newWidget = $entry;
   }
   elsif($_[6] eq 'optionmenu')
   {
      my $val = $cfgHash{$_[7]};
      my @list = split(/:/,$_[8]);
      my $combo = new Gtk2::Combo();
      $combo->set_popdown_strings(@list);

      $combo->set_use_arrows( $false );
      $combo->set_value_in_list( $true, $false );
      $combo->entry->set_text( $val );
      ###debug
      # debug ("OPTIONMENU: $_[7]: $val\n");
      ###end
      #get new values at an arbitary time
      $combo->entry->signal_connect( "changed", \&setOptEntry, $_[7] );
      $newWidget = $combo;
   }
   else
   {
      die "Error: $_[6]: No such Widget available for creation\n";
   }

   $_[0]->attach($newWidget,$_[2]+1,$_[3]+1,$_[4],$_[5], ['shrink','fill'],
         'shrink', 5, 2);
   return($newWidget);
}

sub clickB
{
   #if a toggle signal is set;
   # check if button is already toggled and set it to  opposite state
   my ($name) = $_[1];

   #debug "\nCHECKBUTTON_BEFORE::$cfgHash{$name}\n\n";

   if($_[0]->active)#if toggled
   {
      $cfgHash{$name} = 1;
      #debug "\n CHECKBUTTON_AFTER $cfgHash{$name}\n\n";
   }
   else
   {
      $cfgHash{$name} = 0;
      #debug "\n CHECKBUTTON_AFTER $cfgHash{$name}\n\n";
   }
}

sub setTextEntry
{
   my($name) = $_[1];

   $cfgHash{$name} = $_[0]->get_text();
   #debug "TEXTENTRY Entry contents: $cfgHash{$name}\n";
}

sub setOptEntry
{
   my ($name) = $_[1];

   $cfgHash{$name} = $_[0]->get_text();
   #debug "OPTIONMENU Entry contents: $cfgHash{$name}\n";
}




sub openERR
{
   #if severity has a value application will kill programm after ok button will be clicked
   my ($title,$msg,$severe) = @_;

   $popUp->destroy() unless (! defined($popUp));
   $errWin->destroy() unless (! defined($errWin));
   $errWin = new Gtk2::Window( "toplevel" );
   $errWin->modify_bg('normal', $greybg);

   $errWin->realize();

   $errWin->set_resizable( TRUE );
   $errWin->set_position('mouse' );
   $errWin->set_title( "An error has occured" );
   $errWin->signal_connect( "destroy", sub { $errWin->destroy(); } );
   $errWin->set_border_width( 10 );
   $errWin->set_default_size(300, 200 );

   my $errframe = new Gtk2::Frame();
   $errframe->set_shadow_type( 'none' );
   $errWin->add($errframe);

   # Add vbox for the two tables
   my $errvbox = new Gtk2::VBox($false, 10 );
   $errframe->add( $errvbox );

   my $tlabel = new Gtk2::Label($title);
   $errvbox->pack_start($tlabel, $true,$true,0);

   my $msglabel = new Gtk2::Label($msg);
   $errvbox->pack_start($msglabel, $true,$true,0);
   # table

   my $errtable = new Gtk2::Table( 1, 3, $false );
   $errtable->set_row_spacings( 0 );
   $errtable->set_col_spacings( 20 );

   my $errbutton = new Gtk2::Button("OK");
   
   if($severe == $true) {  $errbutton->signal_connect( 'clicked', sub { Gtk->exit(0); }); }
   else { $errbutton->signal_connect( 'clicked', sub { $errWin->destroy(); }); }
   
   $errtable->attach_defaults($errbutton,1,2,0,1);

   $errvbox->pack_start( $errtable, $true, $false, 0 );

   $errWin->show_all();
}

#create configuration
sub makeSystemconf()
{
   $systemConf = 0;

   if ($cfgHash{'enableBinFile_t'} == 1) { $systemConf |= (1 << 2); }

   if ($cfgHash{'enableWfdiscFile_t'} == 1) { $systemConf |= (1 << 3); }

   if ($cfgHash{'enableTxtFile_t'} eq 'FULL')
   {
      $systemConf |= (1 << 17) | (1 << 16) | (1 << 4);
   }
   if ($cfgHash{'enableTxtFile_t'} eq 'MORE')
   {
      $systemConf |= (1 << 16) | (1 << 4);
   }
   if ($cfgHash{'enableTxtFile_t'} eq 'YES') { $systemConf |= (1 << 4); }

   if ($cfgHash{'enableClfFile_t'} == 1) { $systemConf |= (1 << 5); }

   if ($cfgHash{'enableSyslog_t'} eq 'FULL')
   {
     $systemConf |= (1 << 9) | (1 << 6) | (1 << 26);
   }
   if ($cfgHash{'enableSyslog_t'} eq 'MORE')
   {
      $systemConf |= (1 << 6) | (1 << 26);
   }
   if ($cfgHash{'enableSyslog_t'} eq 'YES') { $systemConf |= (1 << 26); }

   if ($cfgHash{'logConnectionRequests_t'} eq 'ALL')
   {
      $systemConf |= (1 << 8) | (1 << 7);
   }
   if ($cfgHash{'logConnectionRequests_t'} eq 'INVALIDONLY')
   {
      $systemConf |= (1 << 7);
   }
   if ($cfgHash{'logOutgoingData_t'} == 1) { $systemConf |= (1 << 10); }

   if ($cfgHash{'strictModeEnable_t'} == 1) { $systemConf |= (1 << 11); }

   if ($cfgHash{'verSigEnable_t'} eq 'ALL')
   {
      $systemConf |= (1 << 13) | (1 << 14);
   }
   if ($cfgHash{'verSigEnable_t'} eq 'FRAME') { $systemConf |= (1 << 13); }
   if ($cfgHash{'verSigEnable_t'} eq 'CHANNEL') { $systemConf |= (1 << 14); }

   if ($cfgHash{'absWfdiscPath_t'} == 1) { $systemConf |= (1 << 15); }

   if ($cfgHash{'userGroupPrivileges_t'} eq 'READWRITE')
   {
      $systemConf |= (1 << 19) | (1 << 18);
   }
   if ($cfgHash{'userGroupPrivileges_t'} eq 'READONLY')
   {
      $systemConf |= (1 << 18);
   }
   if ($cfgHash{'worldPrivileges_t'} eq 'READWRITE')
   {
      $systemConf |= (1 << 21) | (1 << 20);
   }
   if ($cfgHash{'worldPrivileges_t'} eq 'READONLY')
   {
      $systemConf |= (1 << 20);
   }
# if ($cfgHash{'processMode_t'} eq 'FIFO') { $systemConf |= (1 << 22); }
#    if ($cfgHash{'processMode_t'} eq 'LIFO') { $systemConf |= (1 << 25); }

   return $systemConf;
}


#
# Retrieve values from systemConf
# -------------------------------------------------------------------
#
sub splitSystemconf()
{

    print "SYSTEMCONF::$systemConf\n----------------";

   if ($systemConf & (1 << 2)) { $cfgHash{'enableBinFile_t'} = 1; }
   else { $cfgHash{'enableBinFile_t'} = 0; }

   if ($systemConf & (1 << 3)) { $cfgHash{'enableWfdiscFile_t'} = 1; }
   else { $cfgHash{'enableWfdiscFile_t'} = 0; }

   if ($systemConf & (1 << 17)) { $cfgHash{'enableTxtFile_t'} = 'FULL'; }
   elsif ($systemConf & (1 << 16)) { $cfgHash{'enableTxtFile_t'} = 'MORE'; }
   elsif ($systemConf & (1 << 4)) { $cfgHash{'enableTxtFile_t'} = 'YES'; }
   else { $cfgHash{'enableTxtFile_t'} = 'NO'; }

   if ($systemConf & (1 << 5)) { $cfgHash{'enableClfFile_t'} = 1; }
   else { $cfgHash{'enableClfFile_t'} = 0; }

   if ($systemConf & (1 << 9)) { $cfgHash{'enableSyslog_t'} = 'FULL'; }
   elsif ($systemConf & (1 << 6)) { $cfgHash{'enableSyslog_t'} = 'MORE'; }
   elsif ($systemConf & (1 << 26)) { $cfgHash{'enableSyslog_t'} = 'YES'; }
   else { $cfgHash{'enableSyslog_t'} = 'NO'; }

   if ($systemConf & (1 << 8)) { $cfgHash{'logConnectionRequests_t'} = 'ALL'; }
   elsif ($systemConf & (1 << 7))
   {
      $cfgHash{'logConnectionRequests_t'} = 'INVALIDONLY';
   }
   else { $cfgHash{'logConnectionRequests_t'} = 'NONE'; }

   if ($systemConf & (1 << 10)) { $cfgHash{'logOutgoingData_t'} = 1; }
   else { $cfgHash{'logOutgoingData_t'} = 0; }

   if ($systemConf & (1 << 11)) { $cfgHash{'strictModeEnable_t'} = 1; }
   else { $cfgHash{'strictModeEnable_t'} = 0; }

   if (($systemConf & (1 << 13)) && ($systemConf & (1 << 14)))
      { $cfgHash{'verSigEnable_t'} = 'ALL'; }
   elsif ($systemConf & (1 << 14)) { $cfgHash{'verSigEnable_t'} = 'CHANNEL'; }
   elsif ($systemConf & (1 << 13)) { $cfgHash{'verSigEnable_t'} = 'FRAME'; }
   else { $cfgHash{'verSigEnable_t'} = 'NONE'; }

   if ($systemConf & (1 << 15)) { $cfgHash{'absWfdiscPath_t'} = 1; }
   else { $cfgHash{'absWfdiscPath_t'} = 0; }

   if ($systemConf & (1 << 19))
      { $cfgHash{'userGroupPrivileges_t'} = 'READWRITE'; }
   elsif ($systemConf & (1 << 18))
      { $cfgHash{'userGroupPrivileges_t'} = 'READONLY'; }
   else { $cfgHash{'userGroupPrivileges_t'} = 'NONE'; }

   if ($systemConf & (1 << 21))
      { $cfgHash{'worldPrivileges_t'} = 'READWRITE'; }
   elsif ($systemConf & (1 << 20))
      { $cfgHash{'worldPrivileges_t'} = 'READONLY'; }
   else { $cfgHash{'worldPrivileges_t'} = 'NONE'; }

# if ($systemConf & (1 << 22)) { $cfgHash{'processMode_t'} = 'FIFO'; }
#    elsif ($systemConf & (1 << 25)) { $cfgHash{'processMode_t'} = 'LIFO'; }
#    else { $cfgHash{'processMode_t'} = 'AS_ARRIVED'; }
}


#change station config values
sub set_values()
{
   my $cmd = '';
   my ($widget,$par) = @_;

   $systemConf = &makeSystemconf;

   ##Debug
   debug "OLD_SYSTEMCONF_VALUE: $oldSystemConf\n";
   debug "SYSTEMCONF_VALUE: $systemConf\n";
   debug "OLD_DEBUG_LEVEL: $oldDebugLevel\n";
   debug "DEBUG_LEVEL: $cfgHash{'debugLevel_t'}\n";
   ##end

   #check for valid input in debugLevel_t Textfield
   if(($cfgHash{'debugLevel_t'} =~ m/(\D)+/)|| ($cfgHash{'debugLevel_t'} eq ""))
   {
      my $ERROR = "!!INVALID Debuglevel!!";
      my $MSG = "VALID Debug Levels: 0-99 altough just 0-9 are in use by now";
      openERR($ERROR,$MSG,$false);
      return 0;
   }

   if ($oldSystemConf != $systemConf)
   {
      debug"\nSET_NEW---SYSCONF---\n";
      $cmd = "$CDcontrol $appsList[@$par[1]][1] $appsList[@$par[1]][3] " . 
             "SCP=$systemConf,@$par[0]";
      debug "$cmd\n";
      open (CMD, "$cmd |") or die "can't execute $cmd)";
      while (<CMD>)
      {
          debug $_;
      }
      close(CMD);
   }

   if ($oldDebugLevel != $cfgHash{'debugLevel_t'})
   {
      debug"\nSET_NEW---DEBUG---\n";
      $cmd = "$CDcontrol $appsList[@$par[1]][1] $appsList[@$par[1]][3] " . 
              "SDL=$cfgHash{'debugLevel_t'},@$par[0]";
      debug "$cmd\n";
      open (CMD, "$cmd |") or die "can't execute $cmd)";
      while (<CMD>)
      {
          debug $_;
      }
      close(CMD);
   }

   return 1;
}

# Send reload station command

sub sendRSF
{
   my $cmd = '';
   my ($widget,$par) = @_;

   $cmd = "$CDcontrol $appsList[@$par[1]][1] $appsList[@$par[1]][3] RSF";
   &sendCommand($cmd);
   $popUp->destroy() unless (! defined($popUp));
   return 1;
}


# Send start, stop or disconnect command

sub command
{
   my $cmd = '';
   my ($widget, $event, $par) = @_;

#  print Dumper(@_);

   $cmd = "$CDcontrol $appsList[@$par[1]][1] $appsList[@$par[1]][3] " . 
           "@$par[2]=@$par[0]";
   &sendCommand($cmd);
   $popUp->destroy() unless (! defined($popUp));
   return 1;
}

sub sendCommand()
{

      my ($ok,$nu,$msg);
    
      debug "$_[0]\n";
      open (CMD, "$_[0] |") or die "can't execute $_[0])";
      while (<CMD>)
      {
          debug $_;
      ($ok,$nu,$msg) = split(' ',$_,3);
      }
      close(CMD);

      chop($msg);
      if(check_perms($msg,$_[0]) == 0) {return 0; }

      return 1;
}


# Add an action to the popup window
sub addAction()
{
   my ($text, $vbox, $station, $appListIndex) = @_;
   my ($nu, $state);


   # read state
   if ($text ne 'Reload Station File')
   {
      ($nu, $state, $nu) = split(" ", $stationArray[$appListIndex]{$station}, 3);
   }

   my $label = new Gtk2::Label( " $text ");
   $label->set_alignment( 0, 0 );

   my $event_box = new Gtk2::EventBox();
   # $event_box->realize;
   $event_box->set_events( 'button_press_mask' );
   $event_box->modify_bg('normal', $white );

   $vbox->pack_start( $event_box, $false, $false, 0 );
   $event_box->add( $label );

   if ($text eq 'Statistics')
   {
      $event_box->signal_connect("button_press_event", \&openDetail,
                                 [$station, $appListIndex, $text]);
   }
   elsif ($text eq 'Configure')
   {
      $event_box->signal_connect("button_press_event", \&openConf,
                                 [$station, $appListIndex, $text]);
   }
   elsif ($text eq 'Stop')
   {
      if ($states[$state] ne 'stopped')
      {
         $event_box->signal_connect("button_press_event", \&command,
                                    [$station, $appListIndex, 'STP']);
      }
      else { $label->modify_fg('normal', $greyInactive); }
   }
   elsif ($text eq 'Start')
   {
      if ($states[$state] eq 'stopped')
      {
         $event_box->signal_connect("button_press_event", \&command,
                                    [$station, $appListIndex, 'STR']);
      }
      else { $label->modify_fg('normal', $greyInactive); }
   }
   elsif ($text eq 'Disconnect')
   {
      if ($states[$state] ne 'stopped' && $states[$state] ne 'unconnected')
      {
         $event_box->signal_connect("button_press_event", \&command,
                                    [$station, $appListIndex, 'DIS']);
      }
      else { $label->modify_fg('normal', $greyInactive); }
   }
   elsif ($text eq 'Reload Station File')
   {
      $event_box->signal_connect("button_press_event", \&sendRSF, 
                                 [$station, $appListIndex] );
   }
   else { $label->modify_fg('normal', $greyInactive); }
}

# Create a popup window with available actions
#

sub openPopUp
{
   my ($name, $nodeType, $appListIndex) = @_;

   $popUp->destroy() unless (! defined($popUp));
   $popUp = new Gtk2::Window( "toplevel" );
## $popUp->modify_bg('normal', $greybg);

   $popUp->set_decorated(FALSE);
   $popUp->realize();
   $popUp->set_resizable( TRUE );
   $popUp->set_position('mouse' );
   $popUp->set_title( "Actions" );
   $popUp->signal_connect( "destroy", sub { $popUp->destroy(); } );
   $popUp->set_border_width( 0 );

   my  $frame = new Gtk2::Frame();
   $frame->set_shadow_type( 'in' );
   $popUp->add($frame);
   my $vbox = new Gtk2::VBox($false, 0 );
   $frame->add($vbox);
   $vbox->pack_start( new Gtk2::Label(" $name "), $false, $false, 0 );
   my  $hseparator = new Gtk2::HSeparator();
   $vbox->pack_start( $hseparator, $false, $false, 0 );
   if ($nodeType == SENDER || $nodeType == RECEIVER)
   {
      &addAction('Reload Station File', $vbox, $name, $appListIndex);
   }
   else
   {
      &addAction('Stop', $vbox, $name, $appListIndex);
      &addAction('Start', $vbox, $name, $appListIndex);
      &addAction('Disconnect', $vbox, $name, $appListIndex);
      $hseparator = new Gtk2::HSeparator();
      $vbox->pack_start( $hseparator, $false, $false, 0 );
      &addAction('Statistics', $vbox, $name, $appListIndex);
      $hseparator = new Gtk2::HSeparator();
      $vbox->pack_start( $hseparator, $false, $false, 0 );
      &addAction('Configure', $vbox, $name, $appListIndex);
   }
   $popUp->show_all();
}

# Callback run when the mouse is clicked on a selected row

sub selection_unmade
{
   $popUp->destroy() unless (! defined($popUp));
}

# Callback run when the mouse is clicked inside the CTree.

sub selection_made
{
   my ( $ctree, $path, $column ) = @_;

   my $lmodel = $ctree->get_model();
   my $iter = $lmodel->get_iter($path);

   my ($name, $node_type, $appListIndex) = $lmodel->get($iter);
   print STDERR "row activated: $name, $node_type, $appListIndex \n";
   # $row_ref is now an array ref to the double-clicked row's data.
 
   next if ($node_type == ROOT);

   openPopUp($name, $node_type, $appListIndex);
   return(1);
}


sub getInternalState()
{
   my ($state, $warn, $err, $delta_rec, $delta_sent) = @_;

   if ($state == STATION_CONNECTED && $err > 0)
      { $state = STATION_HAS_ERRORS; }
   if ($state == STATION_CONNECTED && $warn > 0) 
      { $state = STATION_HAS_WARNINGS; }
   if ($state == STATION_UNCONNECTED && $delta_rec == 0 && $delta_sent == 0) 
      { $state = STATION_IS_CONTROL; }

   return($state);
}

# Query the given instance of CDreceiver or CDSender
#
#WORK_QUERY

sub query_state
{
   my ($station, $state, $type, $port, $a_state, $crs, $cr_warn,
       $cr_err, $warn, $err, $str, @stations, $cmd, $ok,
       $actRow, $actLine, $actNode, $framesRec, $framesSent,
       $cmdFramesRec, $cmdFramesSent);

   my $startRow = 0;
   my $father = undef;
   my $node = undef;
   my $row; # debug info;can be removed safely

   debug "\nQUERY STATE\n";
   $ok = 'no';

   # Send command


   if ($isNotInit)
   {
      $model = Gtk2::TreeStore->new('Glib::String',  # name
                                    'Glib::Uint', # node type
                                    'Glib::Uint', # applist index
                                    'Glib::String', # station type
                                    'Glib::String', # bytes received
                                    'Glib::String', # bytes sent
                                    'Glib::String', # debug txt
                                    'Gtk2::Gdk::Pixbuf' );

#     ( " Station name", " CD type", " Received", " Sent", "Debug" );

      # last param  means root node
      $root = insert_node($model, undef, ["hostname", "", "", "", ""], 
                          $root_icon, ROOT, 0);
      debug "QUERY_STATE::jj:$jj\n";
      $row = 2;

      if ($outView == NOTEBOOK)
      {
         # Create a new notebook, place the position of the tabs
         my $nbTable = new Gtk2::Table(3, 6, $false);
         $scrolled_window->add($nbTable);
         my $notebook = new Gtk2::Notebook();
         $notebook->set_tab_pos('top');
         $nbTable->attach_defaults($notebook, 0, 6, 0, 1);
         $notebook->show();
      }
      elsif ($outView == ICON)
      {
         my $iconView = Gtk2::IconView->new_with_model($model);
         $iconView->set_pixbuf_column(7);
      }
      else
      {
         my $treeview = Gtk2::TreeView->new_with_model($model);
         my $column1 = Gtk2::TreeViewColumn->new();
         $column1->set_title('station name');
         ### first cell for column one
         my $icon = Gtk2::CellRendererPixbuf->new();
         $column1->pack_start($icon, FALSE);
         $column1->set_attributes( $icon, 'pixbuf' => ST_ICON );
         ### second cell for column one
         my $name = Gtk2::CellRendererText->new();
         $column1->pack_start($name, FALSE);
         $column1->set_attributes( $name, 'text' => ST_NAME );
         $column1->set_sort_column_id(ST_NAME);
         $treeview->append_column($column1);
         ## Add second column
         my $column2 = Gtk2::TreeViewColumn->new();
         $column2->set_title("CD type");
         my $cd_type = Gtk2::CellRendererText->new();
         $column2->pack_start($cd_type, FALSE);
         $column2->set_attributes($cd_type, 'text' => ST_TYPE);
         $column2->set_sort_column_id(ST_TYPE);
         $treeview->append_column($column2);
         ## Add third column
         my $column3 = Gtk2::TreeViewColumn->new();
         $column3->set_title("Received");
         my $rcvd = Gtk2::CellRendererText->new();
         $column3->pack_start($rcvd, FALSE);
         $column3->set_attributes($rcvd, 'text' => RCV_BYTES);
         $column3->set_sort_column_id(RCV_BYTES);
         $treeview->append_column($column3);

         $treeview->signal_connect( 'row_activated', \&selection_made );
         $treeview->signal_connect( 'unselect_row', \&selection_unmade );

         $scrolled_window->add($treeview);
      }

      $isNotInit = 0; 
      return 1; 
    }

    # iterate through appsList an create all specified subtree-nodes
    for ($ii = 0;$ii <= $jj; $ii++) # $jj: length of @appsList
    {
        #debug "ii:$ii\n";

        my ($myname,$myIP, $myport) = split(':', $appsList[$ii][3], 3);
        my @mytmp;
        # will hold recognized node type(0=receiver,1=sender,2=root)
        my $tmptype = 99;

        $row = 2; # debug info;can be removed safely

        @mytmp = ["$myname\@$myIP:$myport", "", "", ""];
     
        #print ":::::::$appsList[$ii][3] \n";
    
        if (!defined($appsList[$ii][0])) 
        {
           open (CMD,"$CDcontrol $appsList[$ii][1] $appsList[$ii][3] GST |") or 
                die "ERROR while sending command\n";
    
           # identify sender and receiver and insert them into tree
           while (<CMD>) 
           {
              if($_ =~ /000 OK/) 
              { 
                 (undef, undef, $tmptype, undef) = split(' ', $_, 4); 
                 print "$_\n";
              }
           }
           close(CMD);

           if ($tmptype == RECEIVER)
           {
              debug "RECEIVER\n";
              $appsList[$ii][0] = insert_node($model, $root, @mytmp, 
                                              $receiver_icon, RECEIVER, $ii);
           }
           elsif ( $tmptype == SENDER)
           {
              debug "SENDER\n";
              $appsList[$ii][0] = insert_node($model, $root, @mytmp, 
                                              $sender_icon, SENDER, $ii);
           }
           else { debug "--UNKOWN TYPE OF NODE--\n"; }

           $startRow = $row + $ii; # debug info;can be removed safely
           $appsList[$ii][2] = $startRow; # debug info;can be removed safely
         }#end if defined
    }

    # iterate through all specified subtree nodes and get current stations
    # information for each of the nodes; decide wether a station has
    # already been assigned to the tree or if the station is new and
    # the assignment has to be done
    for ($ii = 0;$ii <= $jj; $ii++) #$jj: length of @appsList
    {
        my $count = 0;
        my $cfgList1 = $appsList[$ii][3];
        my @stationInfo;
      
        # extract sender configuration
        $cmd = "$CDcontrol $appsList[$ii][1] $cfgList1 GST=2";
        debug "$cmd\n";
        open (CMD, "$cmd |") or die "can't execute $cmd)";

        # read station information
        while (<CMD>)
        {
          if ($ok eq 'yes')
          {
             if($_ =~ /000 OK/) { next; }
             chop($_);

             $actLine = $_;
             debug "read in line:: $actLine\n";
             ($station, undef) = split(' ', $actLine, 2);
             $stationInfo[$count] = $actLine;
        
             #set found station to alive
             $station_state{$station} = $alive;
        
             #debug
             debug "STATION STATE of $station marked\n";
             $count++;
          }
          else
          {
             debug "$_";
          }
          if ($_ =~ /^000 OK/) { $ok = 'yes'; }
        }
        close(CMD);
        # $tree->thaw();

        # debug only can be removed safely; used to verify that all
        # found stations were read in
        #my $zz;
        #debug "count::$count\n";
        #for ($zz = 0;$zz < $count; $zz++)
        #{
         #  debug "stationInfo[zz][0]##$stationInfo[$zz]\n";
 
          #$actRow = $appsList[$ii][2];#debug info;can be removed safely
        #}
 
        # each of the formerly detected stations information will
        # be extracted to update them in their subtree; before the
        # update is performed the station is checked for beeing already
        # a known station; if so the update is performed; if not the
        # station is inserted in its subtree; and will be assigned to
        # stationNodes[<actual_subtree>]{<actual_station_name>}{<node>}
        my $oo;
        for ($oo = 0; $oo < $count ;$oo++)
        {
           ($station, $state, $type, $port, $a_state, $crs, $cr_warn,
            $cr_err, $warn, $err, $framesRec, $framesSent, undef, undef,
            $cmdFramesRec, $cmdFramesSent) = split(/ /, $stationInfo[$oo]);

           # set to int. error if wrong state
           if (!defined ($station_codes[$state])) { $state = 0; } 

           # name of the actual station is used as an index
           # in the station array. Remember station values
           $stationArray[$ii]{$station} = $stationInfo[$oo];

           # Decide if a station has already been defined or not.
           # If not, assign the station to its subtree, else update it
           if (! defined($stationNodes[$ii]->{$station}))
           {
              my $parentNode = $appsList[$ii][0];

              #debug "new station::stationNodes[$ii]->{$station}\n";
              $actRow++;

              $actNode = $model->append($parentNode); 
              #debug "new_act_Node::$actNode\n";
              $stationNodes[$ii]{$station}= $actNode;
              #debug "new_actNode_loaded::$stationNodes[$ii]{$station}\n";
           }
           else 
           {
              $actNode = $stationNodes[$ii]{$station};
              #debug "Update node station[$ii][$station]\n";
           }

           $state = &getInternalState($state, $cr_warn + $warn, $cr_err + $err,
                                      $framesRec - $cmdFramesRec,
                                      $framesSent - $cmdFramesSent);
           debug "ii:$ii instate:$state\n";
           # update the model data
           $model->set($actNode, ST_ICON, $station_codes[$state][0],
                                 NODE_TYPE, STATION,
                                 CFG_INDEX, $ii,
                                 ST_NAME, $station,
                                 ST_TYPE, $types[$type],
                                 RCV_BYTES, $framesRec,
                                 SND_BYTES, $framesSent,
                                 DBG_TXT, $station);

        } # end iterating through stations

        # detect unused leafs and delete them from tree
        for my $k1 (keys %{$stationArray[$ii]})
        {
           # debug ":::::::::::::stationArray->[$ii]->{$k1}\n";

           if ($station_state{$k1} == $alive)
           {
              debug "::::$k1 ALIVE\n";
              # unmark actual station
              $station_state{$k1} = $dead;
           }
           else 
           {
              debug "::::$k1 DEAD\n";
             
              $model->remove($stationNodes[$ii]->{$k1});
             
              delete $stationNodes[$ii]->{$k1};
              delete $station_state{$k1};
              delete $stationArray[$ii]{$k1};
           }
        } # end leaf deletion

        # detect unused nodes and delete them from tree
        my $k2 = 0;
        $k2 = keys (%{$stationNodes[$ii]});
        if( ($k2 == 0) && (defined($appsList[$ii][0])) )
        {
            debug "keys: $k2\n";
            debug "::::::::FOUND EMPTY $ii, Deleting it :::::::::\n";
     
            $model->remove($appsList[$ii][0]);
            delete $appsList[$ii][0];
            delete $stationArray[$ii];
            delete $stationNodes[$ii];
        }
    } # end loop (iterating trough nodes)
    
    return 1;
}

#check for sufficent permissions if calling gdl,gcp commands
sub check_perms {

    my ($message,$cmd) = @_;
    debug "::: $message,$cmd\n";
    if($message eq "not authorized") {

    openERR("!!Insufficent permissions!!", "You are not authorized to send  \n $cmd",$false);
    return 0;
    
    }

    else { return 1; }

}

# build in XPM icons
sub setupPixmaps
{
@station_green_yellow_xpm = (
"16 16 106 2",
"      c None",
".     c #e9e8e6",
"X     c #ece9e5",
"o     c #e8e6e3",
"O     c #e8e5e3",
"+     c #e7e5e2",
"@     c #e8e5e2",
"#     c #eae7e3",
"\$     c #e2e2e1",
"%     c #a19f9d",
"&     c #d9e1ee",
"*     c #00c000",
"=     c #f0f6ff",
"-     c #848281",
";     c #a6a4a0",
":     c #c3cfe6",
">     c #d5e0f3",
",     c #817f7b",
"<     c #a9a7a3",
"1     c #c5d2e8",
"2     c #cbd5e8",
"3     c #7c7b78",
"4     c #abaaa6",
"5     c #c7d3ea",
"6     c #c9d3e5",
"7     c #7e7b78",
"8     c #aeaba7",
"9     c #cbd8ee",
"0     c #c6d0e3",
"q     c #aeada9",
"w     c #cfdcf1",
"e     c #c4cee1",
"r     c #7b7976",
"t     c #b3b0ad",
"y     c #d5e1f4",
"u     c #c5d0e0",
"i     c #7b7a77",
"p     c gray56",
"a     c white",
"s     c #eff5fc",
"d     c #e7edf4",
"f     c #dfe5eb",
"g     c #dbe2e7",
"h     c #d9dfe5",
"j     c #d5dbe1",
"k     c #d1d6dd",
"l     c #ced3da",
"z     c #ced4db",
"x     c #cfd5dd",
"c     c #dcdedf",
"v     c #6a6a6a",
"b     c #606060",
"n     c #636361",
"m     c #7f7c7c",
"M     c #c5c4c2",
"N     c #dcdbd9",
"B     c #d5d4d2",
"V     c #d3d1d0",
"C     c #d2d0cf",
"Z     c #b1b1af",
"A     c #6d6d6a",
"S     c #565655",
"D     c gray32",
"F     c #e1e1e1",
"G     c #eeeeee",
"H     c #e6e6e6",
"J     c gray92",
"K     c gray91",
"L     c #d8d8d8",
"P     c #e7e7e7",
"I     c #dadada",
"U     c #eaeaea",
"Y     c #f8f8f8",
"T     c #f1f1f1",
"R     c #e7e7e6",
"E     c white smoke",
"W     c gray90",
"Q     c gray34",
"!     c #7a7a78",
"~     c #b8b8b9",
"^     c #a8a8a9",
"/     c #787876",
"(     c #575755",
")     c #bababd",
"_     c #89898c",
"`     c #a8a8b2",
"'     c #c5c5ce",
"]     c #b2b2bb",
"[     c #c2c2c6",
"{     c #dddddc",
"}     c #dcdcdb",
"|     c #a9a9af",
" .    c #a2a2ab",
"..    c #c7c7d0",
"X.    c #a2a2ac",
"o.    c #464649",
"O.    c #4e4e4f",
"+.    c #4f4f54",
"@.    c #737377",
"#.    c #78787b",
"\$.    c #7b7b7d",
"%.    c #66666b",
"&.    c #444445",
"*.    c #4e4e51",
"=.    c #434346",
"A.    c #ffe206",
"    . X o O O + + O @ + # \$     ",
"  % & * * * * * * * * * * = -   ",
"  ; : * * * * * * * * * * > ,   ",
"  < 1 * * * A.A.A.A.* * * 2 3   ",
"  4 5 * * * A.A.A.A.* * * 6 7   ",
"  8 9 * * * A.A.A.A.* * * 0 3   ",
"  q w * * * * * * * * * * e r   ",
"  t y * * * * * * * * * * u i   ",
"  p a s d f g h j k l z x c v   ",
"    b n m M N B V C Z A S D     ",
"        F G H J K L P I         ",
"        U Y T P R U E W         ",
"          Q ! ~ ^ / (           ",
"              ) _               ",
"% % % ` ' ] [ { } |  ...X.% % % ",
"o.o.o.o.O.+.@.#.\$.%.&.*.=.o.o.o.");

@station_green_xpm = (
"16 16 105 2",
"      c None",
".     c #e9e8e6",
"X     c #ece9e5",
"o     c #e8e6e3",
"O     c #e8e5e3",
"+     c #e7e5e2",
"@     c #e8e5e2",
"#     c #eae7e3",
"\$     c #e2e2e1",
"%     c #a19f9d",
"&     c #d9e1ee",
"*     c #00c000",
"=     c #f0f6ff",
"-     c #848281",
";     c #a6a4a0",
":     c #c3cfe6",
">     c #d5e0f3",
",     c #817f7b",
"<     c #a9a7a3",
"1     c #c5d2e8",
"2     c #cbd5e8",
"3     c #7c7b78",
"4     c #abaaa6",
"5     c #c7d3ea",
"6     c #c9d3e5",
"7     c #7e7b78",
"8     c #aeaba7",
"9     c #cbd8ee",
"0     c #c6d0e3",
"q     c #aeada9",
"w     c #cfdcf1",
"e     c #c4cee1",
"r     c #7b7976",
"t     c #b3b0ad",
"y     c #d5e1f4",
"u     c #c5d0e0",
"i     c #7b7a77",
"p     c gray56",
"a     c white",
"s     c #eff5fc",
"d     c #e7edf4",
"f     c #dfe5eb",
"g     c #dbe2e7",
"h     c #d9dfe5",
"j     c #d5dbe1",
"k     c #d1d6dd",
"l     c #ced3da",
"z     c #ced4db",
"x     c #cfd5dd",
"c     c #dcdedf",
"v     c #6a6a6a",
"b     c #606060",
"n     c #636361",
"m     c #7f7c7c",
"M     c #c5c4c2",
"N     c #dcdbd9",
"B     c #d5d4d2",
"V     c #d3d1d0",
"C     c #d2d0cf",
"Z     c #b1b1af",
"A     c #6d6d6a",
"S     c #565655",
"D     c gray32",
"F     c #e1e1e1",
"G     c #eeeeee",
"H     c #e6e6e6",
"J     c gray92",
"K     c gray91",
"L     c #d8d8d8",
"P     c #e7e7e7",
"I     c #dadada",
"U     c #eaeaea",
"Y     c #f8f8f8",
"T     c #f1f1f1",
"R     c #e7e7e6",
"E     c white smoke",
"W     c gray90",
"Q     c gray34",
"!     c #7a7a78",
"~     c #b8b8b9",
"^     c #a8a8a9",
"/     c #787876",
"(     c #575755",
")     c #bababd",
"_     c #89898c",
"`     c #a8a8b2",
"'     c #c5c5ce",
"]     c #b2b2bb",
"[     c #c2c2c6",
"{     c #dddddc",
"}     c #dcdcdb",
"|     c #a9a9af",
" .    c #a2a2ab",
"..    c #c7c7d0",
"X.    c #a2a2ac",
"o.    c #464649",
"O.    c #4e4e4f",
"+.    c #4f4f54",
"@.    c #737377",
"#.    c #78787b",
"\$.    c #7b7b7d",
"%.    c #66666b",
"&.    c #444445",
"*.    c #4e4e51",
"=.    c #434346",
"    . X o O O + + O @ + # \$     ",
"  % & * * * * * * * * * * = -   ",
"  ; : * * * * * * * * * * > ,   ",
"  < 1 * * * * * * * * * * 2 3   ",
"  4 5 * * * * * * * * * * 6 7   ",
"  8 9 * * * * * * * * * * 0 3   ",
"  q w * * * * * * * * * * e r   ",
"  t y * * * * * * * * * * u i   ",
"  p a s d f g h j k l z x c v   ",
"    b n m M N B V C Z A S D     ",
"        F G H J K L P I         ",
"        U Y T P R U E W         ",
"          Q ! ~ ^ / (           ",
"              ) _               ",
"% % % ` ' ] [ { } |  ...X.% % % ",
"o.o.o.o.O.+.@.#.\$.%.&.*.=.o.o.o.");

@station_green_red_xpm = (
"16 16 106 2",
"      c None",
".     c #e9e8e6",
"X     c #ece9e5",
"o     c #e8e6e3",
"O     c #e8e5e3",
"+     c #e7e5e2",
"@     c #e8e5e2",
"#     c #eae7e3",
"\$     c #e2e2e1",
"%     c #a19f9d",
"&     c #d9e1ee",
"*     c #00c000",
"=     c #f0f6ff",
"-     c #848281",
";     c #a6a4a0",
":     c #c3cfe6",
">     c #d5e0f3",
",     c #817f7b",
"<     c #a9a7a3",
"1     c #c5d2e8",
"2     c #cbd5e8",
"3     c #7c7b78",
"4     c #abaaa6",
"5     c #c7d3ea",
"6     c #c9d3e5",
"7     c #7e7b78",
"8     c #aeaba7",
"9     c #cbd8ee",
"0     c #c6d0e3",
"q     c #aeada9",
"w     c #cfdcf1",
"e     c #c4cee1",
"r     c #7b7976",
"t     c #b3b0ad",
"y     c #d5e1f4",
"u     c #c5d0e0",
"i     c #7b7a77",
"p     c gray56",
"a     c white",
"s     c #eff5fc",
"d     c #e7edf4",
"f     c #dfe5eb",
"g     c #dbe2e7",
"h     c #d9dfe5",
"j     c #d5dbe1",
"k     c #d1d6dd",
"l     c #ced3da",
"z     c #ced4db",
"x     c #cfd5dd",
"c     c #dcdedf",
"v     c #6a6a6a",
"b     c #606060",
"n     c #636361",
"m     c #7f7c7c",
"M     c #c5c4c2",
"N     c #dcdbd9",
"B     c #d5d4d2",
"V     c #d3d1d0",
"C     c #d2d0cf",
"Z     c #b1b1af",
"A     c #6d6d6a",
"S     c #565655",
"D     c gray32",
"F     c #e1e1e1",
"G     c #eeeeee",
"H     c #e6e6e6",
"J     c gray92",
"K     c gray91",
"L     c #d8d8d8",
"P     c #e7e7e7",
"I     c #dadada",
"U     c #eaeaea",
"Y     c #f8f8f8",
"T     c #f1f1f1",
"R     c #e7e7e6",
"E     c white smoke",
"W     c gray90",
"Q     c gray34",
"!     c #7a7a78",
"~     c #b8b8b9",
"^     c #a8a8a9",
"/     c #787876",
"(     c #575755",
")     c #bababd",
"_     c #89898c",
"`     c #a8a8b2",
"'     c #c5c5ce",
"]     c #b2b2bb",
"[     c #c2c2c6",
"{     c #dddddc",
"}     c #dcdcdb",
"|     c #a9a9af",
" .    c #a2a2ab",
"..    c #c7c7d0",
"X.    c #a2a2ac",
"o.    c #464649",
"O.    c #4e4e4f",
"+.    c #4f4f54",
"@.    c #737377",
"#.    c #78787b",
"\$.    c #7b7b7d",
"%.    c #66666b",
"&.    c #444445",
"*.    c #4e4e51",
"=.    c #434346",
"A.    c red",
"    . X o O O + + O @ + # \$     ",
"  % & * * * * * * * * * * = -   ",
"  ; : * * * * * * * * * * > ,   ",
"  < 1 * * * A.A.A.A.* * * 2 3   ",
"  4 5 * * * A.A.A.A.* * * 6 7   ",
"  8 9 * * * A.A.A.A.* * * 0 3   ",
"  q w * * * * * * * * * * e r   ",
"  t y * * * * * * * * * * u i   ",
"  p a s d f g h j k l z x c v   ",
"    b n m M N B V C Z A S D     ",
"        F G H J K L P I         ",
"        U Y T P R U E W         ",
"          Q ! ~ ^ / (           ",
"              ) _               ",
"% % % ` ' ] [ { } |  ...X.% % % ",
"o.o.o.o.O.+.@.#.\$.%.&.*.=.o.o.o.");

#station with errors
@station_red_green_xpm = (
"16 16 105 2",
"      c None",
".     c #e9e8e6",
"X     c #ece9e5",
"o     c #e8e6e3",
"O     c #e8e5e3",
"+     c #e7e5e2",
"@     c #e8e5e2",
"#     c #eae7e3",
"\$     c #e2e2e1",
"%     c #a19f9d",
"&     c #d9e1ee",
"*     c red",
"=     c #00c000",
"-     c #f0f6ff",
";     c #848281",
":     c #a6a4a0",
">     c #c3cfe6",
",     c #d5e0f3",
"<     c #817f7b",
"1     c #a9a7a3",
"2     c #c5d2e8",
"3     c #cbd5e8",
"4     c #7c7b78",
"5     c #abaaa6",
"6     c #c7d3ea",
"7     c #c9d3e5",
"8     c #7e7b78",
"9     c #aeaba7",
"0     c #cbd8ee",
"q     c #c6d0e3",
"w     c #aeada9",
"e     c #cfdcf1",
"r     c #c4cee1",
"t     c #7b7976",
"y     c #b3b0ad",
"u     c #d5e1f4",
"i     c #c5d0e0",
"p     c #7b7a77",
"a     c gray56",
"s     c white",
"d     c #eff5fc",
"f     c #e7edf4",
"g     c #dfe5eb",
"h     c #dbe2e7",
"j     c #d9dfe5",
"k     c #d5dbe1",
"l     c #d1d6dd",
"z     c #ced3da",
"x     c #ced4db",
"c     c #cfd5dd",
"v     c #dcdedf",
"b     c #6a6a6a",
"n     c #606060",
"m     c #636361",
"M     c #7f7c7c",
"N     c #c5c4c2",
"B     c #dcdbd9",
"V     c #d5d4d2",
"C     c #d3d1d0",
"Z     c #d2d0cf",
"A     c #b1b1af",
"S     c #6d6d6a",
"D     c #565655",
"F     c gray32",
"G     c #e1e1e1",
"H     c #eeeeee",
"J     c #e6e6e6",
"K     c gray92",
"L     c gray91",
"P     c #d8d8d8",
"I     c #e7e7e7",
"U     c #dadada",
"Y     c #eaeaea",
"T     c #f8f8f8",
"R     c #f1f1f1",
"E     c #e7e7e6",
"W     c white smoke",
"Q     c gray90",
"!     c gray34",
"~     c #7a7a78",
"^     c #b8b8b9",
"/     c #a8a8a9",
"(     c #787876",
")     c #575755",
"_     c #bababd",
"`     c #89898c",
"'     c #a8a8b2",
"]     c #c5c5ce",
"[     c #b2b2bb",
"{     c #c2c2c6",
"}     c #dddddc",
"|     c #dcdcdb",
" .    c #a9a9af",
"..    c #a2a2ab",
"X.    c #c7c7d0",
"o.    c #a2a2ac",
"O.    c #4f4f54",
"+.    c #464649",
"@.    c #4e4e4f",
"#.    c #737377",
"\$.    c #78787b",
"%.    c #7b7b7d",
"&.    c #66666b",
"*.    c #434346",
"=.    c #4e4e51",
"    . X o O O + + O @ + # \$     ",
"  % & * = = = = = = = = = - ;   ",
"  : > * * = = = = = = = = , <   ",
"  1 2 * * * = = = = = = = 3 4   ",
"  5 6 * * * * * = = = = = 7 8   ",
"  9 0 * * * * * * = = = = q 4   ",
"  w e * * * * * * * = = = r t   ",
"  y u * * * * * * * * * = i p   ",
"  a s d f g h j k l z x c v b   ",
"    n m M N B V C Z A S D F     ",
"        G H J K L P I U         ",
"        Y T R I E Y W Q         ",
"          ! ~ ^ / ( )           ",
"              _ `               ",
"b b b ' ] [ { } |  ...X.o.b b b ",
"D D O.+.@.O.#.\$.%.&.*.=.*.D D D ");

#unconnected
@station_red_xpm = (
"16 16 105 2",
"      c None",
".     c #e9e8e6",
"X     c #ece9e5",
"o     c #e8e6e3",
"O     c #e8e5e3",
"+     c #e7e5e2",
"@     c #e8e5e2",
"#     c #eae7e3",
"\$     c #e2e2e1",
"%     c #a19f9d",
"&     c #d9e1ee",
"*     c #ff3232",
"=     c #f0f6ff",
"-     c #848281",
";     c #a6a4a0",
":     c #c3cfe6",
">     c #d5e0f3",
",     c #817f7b",
"<     c #a9a7a3",
"1     c #c5d2e8",
"2     c #cbd5e8",
"3     c #7c7b78",
"4     c #abaaa6",
"5     c #c7d3ea",
"6     c #c9d3e5",
"7     c #7e7b78",
"8     c #aeaba7",
"9     c #cbd8ee",
"0     c #c6d0e3",
"q     c #aeada9",
"w     c #cfdcf1",
"e     c #c4cee1",
"r     c #7b7976",
"t     c #b3b0ad",
"y     c #d5e1f4",
"u     c #c5d0e0",
"i     c #7b7a77",
"p     c gray56",
"a     c white",
"s     c #eff5fc",
"d     c #e7edf4",
"f     c #dfe5eb",
"g     c #dbe2e7",
"h     c #d9dfe5",
"j     c #d5dbe1",
"k     c #d1d6dd",
"l     c #ced3da",
"z     c #ced4db",
"x     c #cfd5dd",
"c     c #dcdedf",
"v     c #6a6a6a",
"b     c #606060",
"n     c #636361",
"m     c #7f7c7c",
"M     c #c5c4c2",
"N     c #dcdbd9",
"B     c #d5d4d2",
"V     c #d3d1d0",
"C     c #d2d0cf",
"Z     c #b1b1af",
"A     c #6d6d6a",
"S     c #565655",
"D     c gray32",
"F     c #e1e1e1",
"G     c #eeeeee",
"H     c #e6e6e6",
"J     c gray92",
"K     c gray91",
"L     c #d8d8d8",
"P     c #e7e7e7",
"I     c #dadada",
"U     c #eaeaea",
"Y     c #f8f8f8",
"T     c #f1f1f1",
"R     c #e7e7e6",
"E     c white smoke",
"W     c gray90",
"Q     c gray34",
"!     c #7a7a78",
"~     c #b8b8b9",
"^     c #a8a8a9",
"/     c #787876",
"(     c #575755",
")     c #bababd",
"_     c #89898c",
"`     c #a8a8b2",
"'     c #c5c5ce",
"]     c #b2b2bb",
"[     c #c2c2c6",
"{     c #dddddc",
"}     c #dcdcdb",
"|     c #a9a9af",
" .    c #a2a2ab",
"..    c #c7c7d0",
"X.    c #a2a2ac",
"o.    c #464649",
"O.    c #4e4e4f",
"+.    c #4f4f54",
"@.    c #737377",
"#.    c #78787b",
"$.    c #7b7b7d",
"%.    c #66666b",
"&.    c #444445",
"*.    c #4e4e51",
"=.    c #434346",
"    . X o O O + + O @ + # \$     ",
"  % & * * * * * * * * * * = -   ",
"  ; : * * * * * * * * * * > ,   ",
"  < 1 * * * * * * * * * * 2 3   ",
"  4 5 * * * * * * * * * * 6 7   ",
"  8 9 * * * * * * * * * * 0 3   ",
"  q w * * * * * * * * * * e r   ",
"  t y * * * * * * * * * * u i   ",
"  p a s d f g h j k l z x c v   ",
"    b n m M N B V C Z A S D     ",
"        F G H J K L P I         ",
"        U Y T P R U E W         ",
"          Q ! ~ ^ / (           ",
"              ) _               ",
"v v v ` ' ] [ { } |  ...X.v v v ",
"S S S o.O.+.@.#.\$.%.&.*.=.S S S ");


@station_yellow_xpm = (
"16 16 105 2",
"      c None",
".     c #e9e8e6",
"X     c #ece9e5",
"o     c #e8e6e3",
"O     c #e8e5e3",
"+     c #e7e5e2",
"@     c #e8e5e2",
"#     c #eae7e3",
"\$     c #e2e2e1",
"%     c #a19f9d",
"&     c #d9e1ee",
"*     c #f3ff08",
"=     c #f0f6ff",
"-     c #848281",
";     c #a6a4a0",
":     c #c3cfe6",
">     c #d5e0f3",
",     c #817f7b",
"<     c #a9a7a3",
"1     c #c5d2e8",
"2     c #cbd5e8",
"3     c #7c7b78",
"4     c #abaaa6",
"5     c #c7d3ea",
"6     c #c9d3e5",
"7     c #7e7b78",
"8     c #aeaba7",
"9     c #cbd8ee",
"0     c #c6d0e3",
"q     c #aeada9",
"w     c #cfdcf1",
"e     c #c4cee1",
"r     c #7b7976",
"t     c #b3b0ad",
"y     c #d5e1f4",
"u     c #c5d0e0",
"i     c #7b7a77",
"p     c gray56",
"a     c white",
"s     c #eff5fc",
"d     c #e7edf4",
"f     c #dfe5eb",
"g     c #dbe2e7",
"h     c #d9dfe5",
"j     c #d5dbe1",
"k     c #d1d6dd",
"l     c #ced3da",
"z     c #ced4db",
"x     c #cfd5dd",
"c     c #dcdedf",
"v     c #6a6a6a",
"b     c #606060",
"n     c #636361",
"m     c #7f7c7c",
"M     c #c5c4c2",
"N     c #dcdbd9",
"B     c #d5d4d2",
"V     c #d3d1d0",
"C     c #d2d0cf",
"Z     c #b1b1af",
"A     c #6d6d6a",
"S     c #565655",
"D     c gray32",
"F     c #e1e1e1",
"G     c #eeeeee",
"H     c #e6e6e6",
"J     c gray92",
"K     c gray91",
"L     c #d8d8d8",
"P     c #e7e7e7",
"I     c #dadada",
"U     c #eaeaea",
"Y     c #f8f8f8",
"T     c #f1f1f1",
"R     c #e7e7e6",
"E     c white smoke",
"W     c gray90",
"Q     c gray34",
"!     c #7a7a78",
"~     c #b8b8b9",
"^     c #a8a8a9",
"/     c #787876",
"(     c #575755",
")     c #bababd",
"_     c #89898c",
"`     c #a8a8b2",
"'     c #c5c5ce",
"]     c #b2b2bb",
"[     c #c2c2c6",
"{     c #dddddc",
"}     c #dcdcdb",
"|     c #a9a9af",
" .    c #a2a2ab",
"..    c #c7c7d0",
"X.    c #a2a2ac",
"o.    c #464649",
"O.    c #4e4e4f",
"+.    c #4f4f54",
"@.    c #737377",
"#.    c #78787b",
"\$.    c #7b7b7d",
"%.    c #66666b",
"&.    c #444445",
"*.    c #4e4e51",
"=.    c #434346",
"    . X o O O + + O @ + # \$     ",
"  % & * * * * * * * * * * = -   ",
"  ; : * * * * * * * * * * > ,   ",
"  < 1 * * * * * * * * * * 2 3   ",
"  4 5 * * * * * * * * * * 6 7   ",
"  8 9 * * * * * * * * * * 0 3   ",
"  q w * * * * * * * * * * e r   ",
"  t y * * * * * * * * * * u i   ",
"  p a s d f g h j k l z x c v   ",
"    b n m M N B V C Z A S D     ",
"        F G H J K L P I         ",
"        U Y T P R U E W         ",
"          Q ! ~ ^ / (           ",
"              ) _               ",
"v v v ` ' ] [ { } |  ...X.v v v ",
"S S S o.O.+.@.#.\$.%.&.*.=.S S S ");

@controller_xpm = (
"16 16 104 2",
"      c None",
".     c #e9e8e6",
"X     c #ece9e5",
"o     c #e8e6e3",
"O     c #e8e5e3",
"+     c #e7e5e2",
"@     c #e8e5e2",
"#     c #eae7e3",
"\$     c #e2e2e1",
"%     c #a19f9d",
"&     c #d9e1ee",
"*     c #f8f8f8",
"=     c #f0f6ff",
"-     c #848281",
";     c #a6a4a0",
":     c #c3cfe6",
">     c #d5e0f3",
",     c #817f7b",
"<     c #a9a7a3",
"1     c #c5d2e8",
"2     c #cbd5e8",
"3     c #7c7b78",
"4     c #abaaa6",
"5     c #c7d3ea",
"6     c #c9d3e5",
"7     c #7e7b78",
"8     c #aeaba7",
"9     c #cbd8ee",
"0     c #c6d0e3",
"q     c #aeada9",
"w     c #cfdcf1",
"e     c #c4cee1",
"r     c #7b7976",
"t     c #b3b0ad",
"y     c #d5e1f4",
"u     c #c5d0e0",
"i     c #7b7a77",
"p     c gray56",
"a     c white",
"s     c #eff5fc",
"d     c #e7edf4",
"f     c #dfe5eb",
"g     c #dbe2e7",
"h     c #d9dfe5",
"j     c #d5dbe1",
"k     c #d1d6dd",
"l     c #ced3da",
"z     c #ced4db",
"x     c #cfd5dd",
"c     c #dcdedf",
"v     c #6a6a6a",
"b     c #606060",
"n     c #636361",
"m     c #7f7c7c",
"M     c #c5c4c2",
"N     c #dcdbd9",
"B     c #d5d4d2",
"V     c #d3d1d0",
"C     c #d2d0cf",
"Z     c #b1b1af",
"A     c #6d6d6a",
"S     c #565655",
"D     c gray32",
"F     c #e1e1e1",
"G     c #eeeeee",
"H     c #e6e6e6",
"J     c gray92",
"K     c gray91",
"L     c #d8d8d8",
"P     c #e7e7e7",
"I     c #dadada",
"U     c #eaeaea",
"Y     c #f1f1f1",
"T     c #e7e7e6",
"R     c white smoke",
"E     c gray90",
"W     c gray34",
"Q     c #7a7a78",
"!     c #b8b8b9",
"~     c #a8a8a9",
"^     c #787876",
"/     c #575755",
"(     c #bababd",
")     c #89898c",
"_     c #a8a8b2",
"`     c #c5c5ce",
"'     c #b2b2bb",
"]     c #c2c2c6",
"[     c #dddddc",
"{     c #dcdcdb",
"}     c #a9a9af",
"|     c #a2a2ab",
" .    c #c7c7d0",
"..    c #a2a2ac",
"X.    c #464649",
"o.    c #4e4e4f",
"O.    c #4f4f54",
"+.    c #737377",
"@.    c #78787b",
"#.    c #7b7b7d",
"\$.    c #66666b",
"%.    c #444445",
"&.    c #4e4e51",
"*.    c #434346",
"    . X o O O + + O @ + # \$     ",
"  % & * * * * * * * * * * = -   ",
"  ; : * * * * * * * * * * > ,   ",
"  < 1 * * * * * * * * * * 2 3   ",
"  4 5 * * * * * * * * * * 6 7   ",
"  8 9 * * * * * * * * * * 0 3   ",
"  q w * * * * * * * * * * e r   ",
"  t y * * * * * * * * * * u i   ",
"  p a s d f g h j k l z x c v   ",
"    b n m M N B V C Z A S D     ",
"        F G H J K L P I         ",
"        U * Y P T U R E         ",
"          W Q ! ~ ^ /           ",
"              ( )               ",
"v v v _ ` ' ] [ { } |  ...v v v ",
"S S S X.o.O.+.@.#.\$.%.&.*.S S S ");

@station_black_xpm = (
"16 16 104 2",
"      c None",
".     c #e9e8e6",
"X     c #ece9e5",
"o     c #e8e6e3",
"O     c #e8e5e3",
"+     c #e7e5e2",
"@     c #e8e5e2",
"#     c #eae7e3",
"\$     c #e2e2e1",
"%     c #a19f9d",
"&     c #d9e1ee",
"*     c #565655",
"=     c #f0f6ff",
"-     c #848281",
";     c #a6a4a0",
":     c #c3cfe6",
">     c #d5e0f3",
",     c #817f7b",
"<     c #a9a7a3",
"1     c #c5d2e8",
"2     c #cbd5e8",
"3     c #7c7b78",
"4     c #abaaa6",
"5     c #c7d3ea",
"6     c #c9d3e5",
"7     c #7e7b78",
"8     c #aeaba7",
"9     c #cbd8ee",
"0     c #c6d0e3",
"q     c #aeada9",
"w     c #cfdcf1",
"e     c #c4cee1",
"r     c #7b7976",
"t     c #b3b0ad",
"y     c #d5e1f4",
"u     c #c5d0e0",
"i     c #7b7a77",
"p     c gray56",
"a     c white",
"s     c #eff5fc",
"d     c #e7edf4",
"f     c #dfe5eb",
"g     c #dbe2e7",
"h     c #d9dfe5",
"j     c #d5dbe1",
"k     c #d1d6dd",
"l     c #ced3da",
"z     c #ced4db",
"x     c #cfd5dd",
"c     c #dcdedf",
"v     c #6a6a6a",
"b     c #606060",
"n     c #636361",
"m     c #7f7c7c",
"M     c #c5c4c2",
"N     c #dcdbd9",
"B     c #d5d4d2",
"V     c #d3d1d0",
"C     c #d2d0cf",
"Z     c #b1b1af",
"A     c #6d6d6a",
"S     c gray32",
"D     c #e1e1e1",
"F     c #eeeeee",
"G     c #e6e6e6",
"H     c gray92",
"J     c gray91",
"K     c #d8d8d8",
"L     c #e7e7e7",
"P     c #dadada",
"I     c #eaeaea",
"U     c #f8f8f8",
"Y     c #f1f1f1",
"T     c #e7e7e6",
"R     c white smoke",
"E     c gray90",
"W     c gray34",
"Q     c #7a7a78",
"!     c #b8b8b9",
"~     c #a8a8a9",
"^     c #787876",
"/     c #575755",
"(     c #bababd",
")     c #89898c",
"_     c #a8a8b2",
"`     c #c5c5ce",
"'     c #b2b2bb",
"]     c #c2c2c6",
"[     c #dddddc",
"{     c #dcdcdb",
"}     c #a9a9af",
"|     c #a2a2ab",
" .    c #c7c7d0",
"..    c #a2a2ac",
"X.    c #464649",
"o.    c #4e4e4f",
"O.    c #4f4f54",
"+.    c #737377",
"@.    c #78787b",
"#.    c #7b7b7d",
"\$.    c #66666b",
"%.    c #444445",
"&.    c #4e4e51",
"*.    c #434346",
"    . X o O O + + O @ + # \$     ",
"  % & * * * * * * * * * * = -   ",
"  ; : * * * * * * * * * * > ,   ",
"  < 1 * * * * * * * * * * 2 3   ",
"  4 5 * * * * * * * * * * 6 7   ",
"  8 9 * * * * * * * * * * 0 3   ",
"  q w * * * * * * * * * * e r   ",
"  t y * * * * * * * * * * u i   ",
"  p a s d f g h j k l z x c v   ",
"    b n m M N B V C Z A * S     ",
"        D F G H J K L P         ",
"        I U Y L T I R E         ",
"          W Q ! ~ ^ /           ",
"              ( )               ",
"v v v _ ` ' ] [ { } |  ...v v v ",
"* * * X.o.O.+.@.#.\$.%.&.*.* * * ");


@station_wait_xpm = (
"16 16 88 1",
"     c None",
".    c #e9e8e6",
"X    c #ece9e5",
"o    c #e8e6e3",
"O    c #e8e5e3",
"+    c #e7e5e2",
"@    c #e8e5e2",
"#    c #eae7e3",
"\$    c #e2e2e1",
"%    c #a19f9d",
"&    c #d9e1ee",
"*    c #1c5cbc",
"=    c #f0f6ff",
"-    c #848281",
";    c #a6a4a0",
":    c #c3cfe6",
">    c #d5e0f3",
",    c #817f7b",
"<    c #a9a7a3",
"1    c #c5d2e8",
"2    c #cbd5e8",
"3    c #7c7b78",
"4    c #abaaa6",
"5    c #c7d3ea",
"6    c #c9d3e5",
"7    c #7e7b78",
"8    c #aeaba7",
"9    c #cbd8ee",
"0    c #c6d0e3",
"q    c #aeada9",
"w    c #cfdcf1",
"e    c #c4cee1",
"r    c #7b7976",
"t    c #b3b0ad",
"y    c #d5e1f4",
"u    c #c5d0e0",
"i    c #7b7a77",
"p    c gray56",
"a    c white",
"s    c red",
"d    c #e7edf4",
"f    c #dfe5eb",
"g    c #dbe2e7",
"h    c #d9dfe5",
"j    c #d5dbe1",
"k    c #d1d6dd",
"l    c #ced3da",
"z    c #ced4db",
"x    c #cfd5dd",
"c    c #dcdedf",
"v    c #606060",
"b    c #c5c4c2",
"n    c #dcdbd9",
"m    c #d5d4d2",
"M    c #d3d1d0",
"N    c #d2d0cf",
"B    c #b1b1af",
"V    c #6d6d6a",
"C    c #565655",
"Z    c #eeeeee",
"A    c #e6e6e6",
"S    c gray92",
"D    c gray91",
"F    c #d8d8d8",
"G    c #e7e7e7",
"H    c #dadada",
"J    c #eaeaea",
"K    c white smoke",
"L    c #b8b8b9",
"P    c #575755",
"I    c #bababd",
"U    c #6a6a6a",
"Y    c #a8a8b2",
"T    c #c5c5ce",
"R    c #b2b2bb",
"E    c #dcdcdb",
"W    c #a9a9af",
"Q    c #a2a2ac",
"!    c #464649",
"~    c #4e4e4f",
"^    c #4f4f54",
"/    c #737377",
"(    c #78787b",
")    c #7b7b7d",
"_    c #66666b",
"`    c #444445",
"'    c #4e4e51",
"]    c #434346",
"  .XoOO++O@+#\$  ",
" %&**********=- ",
" ;:**********>, ",
" <1**********23 ",
" 45**********67 ",
" 89**********03 ",
" qw**********er ",
" ty**********ui ",
" pasdfghjklzxcs ",
"  vssbnmMNBVCss ",
"    sZASDFGH s  ",
"    ssJGssKJss  ",
"     ssLssPss   ",
"      sIss s    ",
"UUUYTRssEWssQUUU",
"CCC!~^/()_`']CCC");

@receiver_xpm = (
"16 16 67 2",
"Qt c None",
".1 c #020202",
".V c #030303",
".J c #047b04",
".T c #048504",
".Q c #058405",
".G c #078307",
".S c #080808",
".L c #088708",
".E c #088908",
".U c #0a0a0a",
".y c #0b9b0b",
".B c #0ba00b",
".p c #0c960c",
".x c #0da60d",
".P c #0eae0e",
".v c #0fa00f",
".q c #0fa20f",
".M c #0faf0f",
".D c #0fb10f",
".C c #0fb30f",
".O c #101010",
".w c #10b110",
".F c #10b610",
".g c #117311",
".K c #12be12",
".5 c #151515",
".I c #171717",
".k c #188f18",
".R c #1a1a1a",
".f c #1a7e1a",
".u c #1ab01a",
".t c #1ab81a",
".# c #218921",
".j c #219b21",
".b c #228a22",
".A c #232323",
".o c #24a024",
".l c #259325",
"#. c #2f2f2f",
".N c #303030",
"## c #333333",
".6 c #373737",
".n c #3ba53b",
".z c #3f3f3f",
"#a c #424242",
".H c #444444",
".m c #44b544",
".r c #4c4c4c",
".W c #525252",
".4 c #535353",
".s c #545454",
".X c #5b5b5b",
".9 c #656565",
".e c #65bf65",
".7 c #676767",
".d c #68b968",
".c c #70bd70",
".Y c #7a7a7a",
".8 c #808080",
".h c #80cb80",
".i c #85c885",
".2 c #999999",
".0 c #9b9b9b",
".3 c #a3a3a3",
".Z c #fefefe",
".a c #ffffff",
"QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
"QtQtQt.#.aQtQtQtQtQtQt.a.bQtQtQt",
"QtQtQt.a.cQtQtQtQtQtQt.d.aQtQtQt",
"QtQt.a.a.e.fQtQtQtQt.g.h.a.aQtQt",
"QtQt.i.a.j.kQtQtQtQt.l.m.a.nQtQt",
"QtQt.o.a.p.qQt.r.sQt.t.u.a.vQtQt",
"QtQt.w.a.x.yQt.z.AQt.B.C.a.DQtQt",
"QtQt.E.a.F.GQt.H.IQt.J.K.a.LQtQt",
"QtQtQt.a.MQtQt.N.OQtQt.P.aQtQtQt",
"QtQtQt.Q.aQtQt.R.SQtQt.a.TQtQtQt",
"QtQtQtQtQtQtQt.U.VQtQtQtQtQtQtQt",
"QtQtQtQtQtQtQt.W.XQtQtQtQtQtQtQt",
"QtQtQtQtQtQt.Y.Z.a.0QtQtQtQtQtQt",
"QtQtQtQtQt.1.N.2.3.4.SQtQtQtQtQt",
"QtQtQtQtQt.5.6.7.8.9#.QtQtQtQtQt",
"QtQtQtQtQtQtQt###aQtQtQtQtQtQtQt");


@sender_xpm = (
"16 16 67 2",
"Qt c None",
".1 c #020202",
".U c #030303",
".D c #047b04",
".P c #048504",
".S c #058405",
".K c #078307",
".R c #080808",
".F c #088708",
".I c #088908",
".T c #0a0a0a",
".C c #0b9b0b",
".v c #0ba00b",
".t c #0c960c",
".B c #0da60d",
".L c #0eae0e",
".p c #0fa00f",
".u c #0fa20f",
".O c #0faf0f",
".x c #0fb10f",
".w c #0fb30f",
".N c #101010",
".A c #10b110",
".J c #10b610",
".d c #117311",
".E c #12be12",
".5 c #151515",
".H c #171717",
".m c #188f18",
".Q c #1a1a1a",
".g c #1a7e1a",
".o c #1ab01a",
".n c #1ab81a",
".a c #218921",
".l c #219b21",
".# c #228a22",
".z c #232323",
".s c #24a024",
".h c #259325",
"#. c #2f2f2f",
".M c #303030",
"## c #333333",
".6 c #373737",
".j c #3ba53b",
".y c #3f3f3f",
"#a c #424242",
".G c #444444",
".i c #44b544",
".q c #4c4c4c",
".V c #525252",
".4 c #535353",
".r c #545454",
".W c #5b5b5b",
".9 c #656565",
".f c #65bf65",
".7 c #676767",
".b c #68b968",
".c c #70bd70",
".X c #7a7a7a",
".8 c #808080",
".e c #80cb80",
".k c #85c885",
".2 c #999999",
".0 c #9b9b9b",
".3 c #a3a3a3",
".Y c #fefefe",
".Z c #ffffff",
"QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
"QtQtQtQt.#QtQtQtQtQtQt.aQtQtQtQt",
"QtQtQt.bQtQtQtQtQtQtQtQt.cQtQtQt",
"QtQt.d.eQtQtQtQtQtQtQtQt.f.gQtQt",
"QtQt.h.iQt.jQtQtQtQt.kQt.l.mQtQt",
"QtQt.n.oQt.pQt.q.rQt.sQt.t.uQtQt",
"QtQt.v.wQt.xQt.y.zQt.AQt.B.CQtQt",
"QtQt.D.EQt.FQt.G.HQt.IQt.J.KQtQt",
"QtQtQt.LQtQtQt.M.NQtQtQt.OQtQtQt",
"QtQtQtQt.PQtQt.Q.RQtQt.SQtQtQtQt",
"QtQtQtQtQtQtQt.T.UQtQtQtQtQtQtQt",
"QtQtQtQtQtQtQt.V.WQtQtQtQtQtQtQt",
"QtQtQtQtQtQt.X.Y.Z.0QtQtQtQtQtQt",
"QtQtQtQtQt.1.M.2.3.4.RQtQtQtQtQt",
"QtQtQtQtQt.5.6.7.8.9#.QtQtQtQtQt",
"QtQtQtQtQtQtQt###aQtQtQtQtQtQtQt");


@root_xpm = (
"16 16 36 1",
". c None",
"H c #000000",
"E c #282828",
"b c #41519f",
"G c #464646",
"y c #484848",
"z c #4b4b4b",
"A c #4d4d4d",
"q c #4f4f4f",
"F c #505050",
"B c #575757",
"# c #687eba",
"k c #73787f",
"g c #74797f",
"s c #757575",
"v c #7e7e7e",
"f c #a7c3f0",
"p c #b4b4b4",
"w c #b5b5b5",
"u c #b6b6b6",
"x c #bcbcbc",
"D c #cecece",
"C c #e1e1e1",
"o c #e4efff",
"n c #e5f0ff",
"r c #e6e6e6",
"m c #e6f0ff",
"l c #e6f1ff",
"j c #e7f0ff",
"h c #e8f1ff",
"i c #e8f2ff",
"t c #e9e9e9",
"e c #e9f2ff",
"d c #ebf3ff",
"c c #eef5ff",
"a c #ffffff",
".....######.....",
"....#aaaaaab....",
"....#accdefb....",
"....#accghfb....",
"....#accijfb....",
"....#accklfb....",
"....#accmnfb....",
"....#acccofb....",
"....#acccofb....",
"....#acccofb....",
"....#afffffb....",
".....bbpqbb.....",
".......qq.......",
"rpppppstuvwxpppp",
"yzAAABCDxwEBFyyG",
"......HHHH......");
}
