#!/usr/bin/perl -w
#
#  Script to generate restart files
#
#  ----------------------------------------------------------
#  Author: Gerald Klinkl, PSE KB22, gerald.klinkl@siemens.com
#  Last modification $Date: 2008-03-20 17:11:13 $
#  $Revision: 1.2 $
#  ----------------------------------------------------------
#  Script to generate restart files using clf files created
#  by the peers CD receiver or local clf file created by the
#  sender and local clf files used by CD Sender
#
#  Usage: createRestart.pl production_line config_file
#         last_clf_file_received
#  ----------------------------------------------------------
#  ToDo:
#  ----------------------------------------------------------
#
#  $Log: createRestart.pl,v $
#  Revision 1.2  2008-03-20 17:11:13  gerald
#  Improve index file scanning
#
#  Revision 1.1.1.1  2006/01/31 12:34:32  gerald
#  move libs to gbase
#
#

use strict;
my $debug = 1;

if (@ARGV != 3)
{ die "usage: $0 production_line config_file last_clf_file_received\n";}

my $prodLine = shift(@ARGV);
my $configFile = shift(@ARGV);
my $clfFile = shift(@ARGV);

#------------------------------------------------------------
### Subroutines
#------------------------------------------------------------
sub debug
{
   print "DEBUG: @_" if $debug;
}

#------------------------------------------------------------
# Extract station name
#------------------------------------------------------------
my $station = $clfFile;
$station =~ s/^.*\///;
$station =~ s/\..+$//;

die "Production line [$prodLine] does not match file name of" .
    " clf file [$clfFile]\n" unless $prodLine =~ /${station}_?/o;

open(CLF,"<$clfFile") or die "can't open $clfFile\n";
open(CONFIG,"<$configFile") or die "can't open $configFile\n";

my ($lastLine, $idxPath, $refStorePath);
my $port = 8000;
my $mode = 'AS_ARRIVED';
my %modes = ( AS_ARRIVED => 0, FIFO => 1, LIFO => 2 );

#------------------------------------------------------------
# Search for index file path
#------------------------------------------------------------
while (<CONFIG>) {
  if (/^cdrecvIndexFilePath=(\S+)/) { $idxPath = $1; next; }
  if (/^port=(\d+)/) { $port = $1; next; }
# if (/^processMode=(\S+)/) { $mode = $1; next; }
  if (/^refListStorePath=(\S+)/) { $refStorePath = $1; next; }
}
close CONFIG;
die "No cdrecvIndexFilePath found in config file [$configFile]"
    unless defined($idxPath);
die "No refListStorePath found in config file [$configFile]"
    unless defined($refStorePath);
&debug("indexFilePath: $idxPath\n");
&debug("port: $port\n");
&debug("processMode: $mode\n");
&debug("refListStorePath: $refStorePath\n");

#------------------------------------------------------------
# Search for last DF sent
#------------------------------------------------------------
while (<CLF>) {
  next unless />d/i;
  $lastLine = $_;
}
close CLF;
die "No data frame found in clf file [$clfFile]" unless defined($lastLine);
&debug("Last DF received: $lastLine");

#------------------------------------------------------------
# Extraxt clf files from index files
#------------------------------------------------------------
my @lastLine = split (' ', $lastLine);
my @currentLine;
my $restartFile;

open GREP, "fgrep $station $idxPath/_$port.* |" or die "can't run fgrep";
while (<GREP>) {
   chop;
   s/=.+//;
   s/.*\.idx://;
   $restartFile = "$_.clf";
   &debug("process $restartFile\n");

   #------------------------------------------------------------
   # search for last frame sent in clf file
   #------------------------------------------------------------
   open(CLF,"<$restartFile") or die "can't open $restartFile\n";
   while (<CLF>) {
      @currentLine = split;
      if ($currentLine[3] eq $lastLine[3] && $currentLine[6] eq $lastLine[6])
      {
         # Last frame sent found. Create restart file now.
         # This is only a simple approach, so there might be the case
         # that too many frames are resend in LIFO or FIFO mode. But
         # otherwise the rest of the file have to be scanned for frames
         # with nominal times greater than or less than the nominal time
         # of the last frame sent and the receiving clf file has to be
         # scanned for each frame found to adjust the 'last clf line parsed'
         # line. The are no last N frames resent, because the last frame
         # received is known.
         $prodLine = "$refStorePath/.$prodLine.irl";
         open(RES,">$prodLine") or die "can't open $restartFile\n";
         print RES "$modes{$mode} $restartFile $currentLine[2]\n";
         print RES "$restartFile $currentLine[2] $currentLine[6]\n";
         print RES "__END__\n";
         last;
      }
   }
   close CLF;
}
close GREP;
