/*
 * FILE NAME  idcstreamlog_cookie.h
 *
 *
 * SYNOPSIS
 *      Prototypes and constants for the libidcstreamlog.a clients.
 *
 * AUTHORS
 *
 *      Martin Ertl, Jan Kianicka (2015-11-21)
 *
 * MODIFICATIONS
 *
 */

#ifndef IDCSTREAMLOG_COOKIE_H
#define IDCSTREAMLOG_COOKIE_H

/*
 * Preferred order of includes,
 * according to the Google Style Guide:
 *
 *      1. Public .h file we are extending (if any).
 * 	2. C system files.
 * 	3. Other libraries' .h files.
 * 	4. Our project's .h files.
 *
 * Order files alphabetically within each section.
 */

#include "idcstreamlog_logger.h" /* idcstreamlog_logger_t */
#include "idcstreamlog_core.h"   /* idcstreamlog_severity_t */

/*** Type definitions *******************************************************/

/**
 * Abstract base class.
 */
typedef struct idcstreamlog_cookie_       idcstreamlog_cookie_t;
typedef struct idcstreamlog_cookie_class_ idcstreamlog_cookie_class_k;

/*** IdcStreamLog_Cookie class **********************************************/

/* Structure definitions */

/**
 * Virtual function table (vtable) structure.
 */
struct idcstreamlog_cookie_class_
{
    /* Creation functions */
    idcstreamlog_cookie_t *(*new)(char const **topic,
				  idcstreamlog_severity_t severity);
    idcstreamlog_cookie_t *(*bless)(idcstreamlog_cookie_t *self);
    idcstreamlog_cookie_t *(*construct)(idcstreamlog_cookie_t *self,
					char const **topic,
					idcstreamlog_severity_t severity);
    idcstreamlog_cookie_t *(*destruct)(idcstreamlog_cookie_t *self);
    /* Builder functions */
    void (*add_logger)(idcstreamlog_cookie_t *self, idcstreamlog_logger_t *l);
    void (*unlink_logger)(idcstreamlog_cookie_t *self,
			  idcstreamlog_logger_t *l);
    /* Accessor/setter functions */
    idcstreamlog_logger_t *(*find_logger)(idcstreamlog_cookie_t *self,
					  idcstreamlog_logger_t *lin);
    /* Service functions */
};

/* Global object declarations */

/**
 * Global class object.
 */
extern idcstreamlog_cookie_class_k IdcStreamLog_Cookie;

/* Interface (selector) function declarations */

/**
 * Virtual destructor.
 */
idcstreamlog_cookie_t *
idcstreamlog_cookie_destruct(idcstreamlog_cookie_t *self);

/**
 * Polymorphic de-allocator.
 */
void idcstreamlog_cookie_delete(idcstreamlog_cookie_t **self);

/**
 * Builder functions.
 */
void idcstreamlog_cookie_add_logger(idcstreamlog_cookie_t *self,
				    idcstreamlog_logger_t *l);
void idcstreamlog_cookie_unlink_logger(idcstreamlog_cookie_t *self,
				       idcstreamlog_logger_t *l);

/**
 * Accessor/setter functions.
 */
idcstreamlog_logger_t *
idcstreamlog_cookie_find_logger(idcstreamlog_cookie_t *self,
				idcstreamlog_logger_t *lin);

/**
 * Service functions.
 */

/* Interface function declarations */

#endif
