/*
 * FILE NAME  idcstreamlog_stdtopic.h
 *
 *
 * SYNOPSIS
 *      Prototypes and constants for the libidcstreamlog.a clients.
 *
 * AUTHORS
 *
 *      Martin Ertl, Jan Kianicka (2015-11-21)
 *
 * MODIFICATIONS
 *
 */

#ifndef IDCSTREAMLOG_STDTOPIC_H
#define IDCSTREAMLOG_STDTOPIC_H

/*
 * Preferred order of includes,
 * according to the Google Style Guide:
 *
 *      1. Public .h file we are extending (if any).
 * 	2. C system files.
 * 	3. Other libraries' .h files.
 * 	4. Our project's .h files.
 *
 * Order files alphabetically within each section.
 */

#include "idcstreamlog_topic.h"    /* Base class */

#include <stdio.h>                 /* FILE */

#include "idcstreamlog_logger.h"   /* idcstreamlog_logger_t */
#include "idcstreamlog_systopic.h" /* idcstreamlog_systopic_t */
#include "idcstreamlog_core.h"     /* idcstreamlog_severity_t */

/*** Type definitions *******************************************************/

/**
 * Base types.
 */
typedef struct idcstreamlog_stdtopic_       idcstreamlog_stdtopic_t;
typedef struct idcstreamlog_stdtopic_class_ idcstreamlog_stdtopic_class_k;

/*** IdcStreamLog_StdTopic class ********************************************/

/* Structure definitions */

/**
 * Virtual function table (vtable) structure.
 */
struct idcstreamlog_stdtopic_class_
{
    /* Creation functions */
    idcstreamlog_stdtopic_t *(*new)(char const *name,
				    idcstreamlog_systopic_t *st);
    idcstreamlog_stdtopic_t *(*bless)(idcstreamlog_stdtopic_t *self);
    idcstreamlog_topic_t *(*construct)(idcstreamlog_stdtopic_t *self,
				       char const *name,
				       idcstreamlog_systopic_t *st);
    idcstreamlog_topic_t *(*destruct)(idcstreamlog_stdtopic_t *self);
    /* Builder functions */
    void (*add_logger)(idcstreamlog_topic_t *self,
		       idcstreamlog_logger_t *l,
		       idcstreamlog_severity_t smin,
		       idcstreamlog_severity_t smax);
    void (*copy_logger)(idcstreamlog_topic_t *self, idcstreamlog_logger_t *l,
			idcstreamlog_topic_t *proto);
    void (*unlink_logger)(idcstreamlog_topic_t *self,
			  idcstreamlog_logger_t *l);
    /* Accessor/setter functions */
    idcstreamlog_logger_t * (*find_logger)(idcstreamlog_topic_t *self,
					   idcstreamlog_logger_t *lin);
    /* Service functions */
    void (*open)(idcstreamlog_stdtopic_t *self);
    void (*close)(idcstreamlog_stdtopic_t *self);
};

/**
 * Object structure.
 */
struct idcstreamlog_stdtopic_
{
    idcstreamlog_topic_t super; /* base class; must be the first entry */
    /**
     * Public attributes
     */
    FILE *err;
    FILE *out;
    FILE *off;
    idcstreamlog_systopic_t *systopic; /* @todo(ME) should systopic be private? */
};

/* Global object declarations */

/**
 * Global class object.
 */
extern idcstreamlog_stdtopic_class_k IdcStreamLog_StdTopic;

/* Interface function declarations */

/**
 * Caster functions.
 */
idcstreamlog_topic_t *
idcstreamlog_stdtopic_to_topic(idcstreamlog_stdtopic_t *self);

/**
 * Accessor/setter functions.
 */
FILE *idcstreamlog_stdtopic_get_stream(idcstreamlog_stdtopic_t *self,
				       idcstreamlog_stdstream_t s);
void idcstreamlog_stdtopic_set_stream(idcstreamlog_stdtopic_t *self,
				      idcstreamlog_stdstream_t s, FILE *stream);
idcstreamlog_systopic_t *
idcstreamlog_stdtopic_get_systopic(idcstreamlog_stdtopic_t *self);

#endif
