/*
 * FILE NAME  idcstreamlog_syslogger.h
 *
 *
 * SYNOPSIS
 *      Prototypes and constants for the libidcstreamlog.a clients.
 *
 * AUTHORS
 *
 *      Martin Ertl, Jan Kianicka (2015-11-19)
 *
 * MODIFICATIONS
 *
 */

#ifndef IDCSTREAMLOG_SYSLOGGER_H
#define IDCSTREAMLOG_SYSLOGGER_H

/*
 * Preferred order of includes,
 * according to the Google Style Guide:
 *
 *      1. Public .h file we are extending (if any).
 * 	2. C system files.
 * 	3. Other libraries' .h files.
 * 	4. Our project's .h files.
 *
 * Order files alphabetically within each section.
 */

#include "idcstreamlog_logger.h" /* Base class */

#include <stdarg.h>              /* va_list */

#include "idcstreamlog_core.h"   /* idcstreamlog_severity_t */

/*** Type definitions *******************************************************/

typedef struct idcstreamlog_syslogger_       idcstreamlog_syslogger_t;
typedef struct idcstreamlog_syslogger_class_ idcstreamlog_syslogger_class_k;

/*** IdcStreamLog_SysLogger class *******************************************/

/* Structure definitions */

/**
 * Virtual function table (vtable) structure.
 */
struct idcstreamlog_syslogger_class_
{
    /* Creation functions */
    idcstreamlog_syslogger_t *(*new)(char const *name, char const *syslogid);
    idcstreamlog_syslogger_t *(*bless)(idcstreamlog_syslogger_t *self);
    idcstreamlog_syslogger_t *(*construct)(idcstreamlog_syslogger_t *self,
					   char const *name,
					   char const *syslogid);
    idcstreamlog_logger_t *(*destruct)(idcstreamlog_logger_t *self);
    /* Builder functions */
    /* Accessor/setter functions */
    /* Service functions */
    void (*open)(idcstreamlog_syslogger_t *self);
    void (*close)(idcstreamlog_syslogger_t *self);
    void (*write)(idcstreamlog_syslogger_t *self,
		  char const *topic, idcstreamlog_severity_t,
		  char const *fmt, va_list ap);
    void (*rotate)(idcstreamlog_logger_t *self);
};

/* Global object declarations */

/**
 * Global class object.
 */
extern idcstreamlog_syslogger_class_k IdcStreamLog_SysLogger;

#endif
