/*
 * FILE NAME  idcstreamlog_systopic.h
 *
 *
 * SYNOPSIS
 *      Prototypes and constants for the libidcstreamlog.a clients.
 *
 * AUTHORS
 *
 *      Martin Ertl, Jan Kianicka (2015-11-19)
 *
 * MODIFICATIONS
 *
 */

#ifndef IDCSTREAMLOG_SYSTOPIC_H
#define IDCSTREAMLOG_SYSTOPIC_H

/*
 * Preferred order of includes,
 * according to the Google Style Guide:
 *
 *      1. Public .h file we are extending (if any).
 * 	2. C system files.
 * 	3. Other libraries' .h files.
 * 	4. Our project's .h files.
 *
 * Order files alphabetically within each section.
 */

#include "idcstreamlog_topic.h"  /* Base class */

#include <stdarg.h>              /* va_list */
#include <stdio.h>               /* FILE */

#include "idcstreamlog_logger.h" /* idcstreamlog_logger_t */
#include "idcstreamlog_core.h"   /* idcstreamlog_severity_t */

/*** Type definitions *******************************************************/

/**
 * Base types.
 */
typedef struct idcstreamlog_systopic_       idcstreamlog_systopic_t;
typedef struct idcstreamlog_systopic_class_ idcstreamlog_systopic_class_k;

/*** IdcStreamLog_SysTopic class ********************************************/

/* Structure definitions */

/**
 * Virtual function table (vtable) structure.
 */
struct idcstreamlog_systopic_class_
{
    /* Creation functions */
    idcstreamlog_systopic_t *(*new)(char const *name);
    idcstreamlog_systopic_t *(*bless)(idcstreamlog_systopic_t *self);
    idcstreamlog_topic_t *(*construct)(idcstreamlog_systopic_t *self,
				       char const *name);
    idcstreamlog_topic_t *(*destruct)(idcstreamlog_systopic_t *self);
    /* Builder functions */
    void (*add_logger)(idcstreamlog_systopic_t *self,
		       idcstreamlog_logger_t *l,
		       idcstreamlog_severity_t smin,
		       idcstreamlog_severity_t smax);
    void (*copy_logger)(idcstreamlog_systopic_t *self,
			idcstreamlog_logger_t *l,
			idcstreamlog_systopic_t *proto);
    void (*unlink_logger)(idcstreamlog_systopic_t *self,
			  idcstreamlog_logger_t *l);
    /* Accessor/setter functions */
    idcstreamlog_logger_t *(*find_logger)(idcstreamlog_systopic_t *self,
					  idcstreamlog_logger_t *lin);
    /* Service functions */
    void (*open)(idcstreamlog_systopic_t *self);
    void (*close)(idcstreamlog_systopic_t *self);
};

/**
 * Object structure.
 */
struct idcstreamlog_systopic_
{
    idcstreamlog_topic_t super; /* base class; must be the first entry */
    idcstreamlog_severity_t default_log_upto;
    /**
     * Public attributes
     */
    FILE *emg;
    FILE *art;
    FILE *crt;
    FILE *err;
    FILE *wrn;
    FILE *ntc;
    FILE *ifo;
    FILE *dbg;
    FILE *dg1;
    FILE *dg2;
    FILE *dg3;
    FILE *off;
};

/* Global object declarations */

/**
 * Global class object.
 */
extern idcstreamlog_systopic_class_k IdcStreamLog_SysTopic;

/* Interface function declarations */

/**
 * Caster functions.
 */
idcstreamlog_topic_t *
idcstreamlog_systopic_to_topic(idcstreamlog_systopic_t *self);

/**
 * Accessor/setter functions.
 */
FILE *idcstreamlog_systopic_get_stream(idcstreamlog_systopic_t *self,
				       idcstreamlog_severity_t s);
void idcstreamlog_systopic_set_stream(idcstreamlog_systopic_t *self,
				      idcstreamlog_severity_t s, FILE *stream);

/**
 * Service functions.
 */
void idcstreamlog_systopic_write(idcstreamlog_systopic_t *self,
				 idcstreamlog_severity_t s, char const *buf);

#endif
