#ifndef _COMMAND_H
#define _COMMAND_H

/**
 * @author    Gerald Klinkl
 *
 * Revision 1.14  2013-04-03 gerald
 * Fix include dependencies
 */

#include "cd_common.h"

/* ==== Defines ============================================================= */

#define CANNOT_SET_SHUTDOWN_FLAG(pS)  (STATION_UNCONNECTED == (pS)->state || \
                                       STATION_STOPPED == (pS)->state)

/* ==== Defines for command ================================================= */

#define GET_STATISTICS          0       /* Get monitor information */
#define SET_CONFIG_PARAMS       1       /* Change sysConf value */
#define DISCONNECT_STATION      2       /* Drop active connection */
#define RELOAD_STATION_FILE     3       /* Reload station file */
#define SET_DEBUG_LEVEL         4       /* Change debug level */
#define UPDATE_CERT_CACHE       5       /* Update Certificate Cache */
#define GET_DEBUG_LEVEL         6       /* Get debug level */
#define GET_CONFIG_PARAMS       7       /* Get sysConf value */
#define START_STATION           8       /* start station */
#define STOP_STATION            9       /* stop station */
#define APPLICATION_DEF_1       20      /* application specific command 1 */
#define APPLICATION_DEF_2       21      /* application specific command 2 */
#define APPLICATION_DEF_3       22      /* application specific command 3 */
#define APPLICATION_DEF_4       23      /* application specific command 4 */
#define APPLICATION_DEF_5       24      /* application specific command 5 */
#define APPLICATION_DEF_6       25      /* application specific command 6 */

/* ==== Defines for replies ================================================= */

#define REPLY_OK      "000 OK\n"
#define REPLY_QUEUED  "001 OK command queued\n"
#define REPLY_E100    "100 internal error\n"
#define REPLY_E101    "101 station not authorized\n"
#define REPLY_E102    "102 invalid command size\n"
#define REPLY_E103    "103 Connection ID required\n"
#define REPLY_E104    "104 Connection ID not found\n"
#define REPLY_E105    "105 No station file configured\n"
#define REPLY_E106    "106 Re-loading station file failed\n"
#define REPLY_E107    "107 no command registered\n"
#define REPLY_E108    "108 unknown command\n"
#define REPLY_E109    "109 unknown command parameter\n"
#define REPLY_E110    "110 not connected\n"
#define REPLY_E111    "111 value out of range\n"
#define REPLY_E112    "112 not in stopped state\n"
#define REPLY_E113    "113 already in stopped state\n"
#define REPLY_E114    "114 command too old\n"
#define REPLY_E115    "115 only allowed in global command\n"
#define REPLY_E116    "116 only allowed in station command\n"

/* ==== Function prototypes ================================================= */

extern uint16_t
checkCommandPayload(char *buffer, station_t *pStation,
                    /*@out@*/CDcomReq_t *payload);
extern uint32_t executeCommand(char *buffer, int bufferSize,
                               station_t *pListen, station_t *pStation);
extern int checkCommand(const char *command);
extern int checkCommandAndSecLevel(CDheader_t *pHeader, station_t *pStation,
                                   const char *command);
extern /*@null@*//*@dependent@*/ station_t
       *getControlObject(configParams_t *pConfigData);
extern void printCommands();
extern /*@null@*/ void * commandThread(void *arg);

#endif /* _COMMAND_H */
