#ifndef _EVENTS_H
#define _EVENTS_H

/**
 * @internal
 * @author    Gerald Klinkl
 *
 * Revision 1.35  2014-02-21 GK
 * Add error message for 'invalid frame header detected'
 * when parsing stored (not received) binary files
 *
 */

#include "cd_common.h"

/* ==== Defines and data structures for events ============================== */

/**
 * Event description structure
 *
 * This structure is used to store message text and priority for an event
 */
typedef struct _CDeventTypes {
   const char *msg;      /**< event text */
   int        logPriority; /**< event priority */
} CDeventTypes_t;

#define         NO         ((unsigned char) 0) /* must be 0 !! */
#define         YES        ((unsigned char) 1)

#define         NO_INFO        0 /**< No additional information */

#define         ET_INTERNAL   0
#define         ET_E_CR       1   /**< connection request error */
#define         ET_W_CR       2   /**< connection request warning */
#define         ET_S_CR       3   /**< connection request info */
#define         ET_E_CON      4   /**< error in connected state */
#define         ET_W_CON      5   /**< warning in connected state */
#define         ET_S_CON      6   /**< info in connected state */
#define         ET_L_ERROR    1   /**< error in connected/unconnected state */
#define         ET_L_WARNING  2   /**< warning in connected/unconnected state */
#define         ET_L_INFO     3   /**< info in connected/unconnected state */

#define         NO_ERROR           0
#define         E_IP_UNKNOWN       1
#define         E_TIMEOUT          2
#define         E_FRAMESIZE        3
#define         S_CRF_RECEIVED     4
#define         E_NO_PORT          5
#define         E_NO_MEM           6
#define         E_THREAD           7
#define         E_ISCONNECTED      8
#define         E_THREAD2SLOW      9
#define         S_CONNECTED        10
#define         E_RESPONSE         11
#define         E_FILES            12
#define         E_SYSTEM           13
#define         S_CRF_ACCEPTED     14
#define         S_SEND_RESPONSE    15
#define         E_STATION_UNKNOWN  16
#define         S_DATA             17
#define         S_DATA_FORMAT      18
#define         S_ALERT            19
#define         E_NO_DFF           20
#define         S_SHUTDOWN         21
#define         E_FRAME_TYPE       22
#define         E_FRAME_TOO_BIG    23
#define         E_SEND_TIMEOUT     24
#define         S_SEND_ALERT       25
#define         E_INVALID_OPTION_TYPE 26
#define         E_MAX_CONNECTIONS  27
#define         E_DFF_SIZE         28
#define         E_DF_SIZE          29
#define         E_CHANNEL          30
#define         E_CRC              31
#define         E_ALERT            32
#define         S_CLOSED_BY_AF     33
#define         S_FLUSH_FORMAT     34
#define         E_AUTHSWITCH       35
#define         E_COMPRESSSW       36
#define         E_NOT_ZEROED       37
#define         E_NO_FRAME_SIG     38
#define         E_NO_MEM1          39       /* No mem for frame */
#define         E_NO_MEM2          40       /* No mem for DFinfo */
#define         E_NO_MEM3          41       /* No mem for formatframe */
#define         E_NO_MEM4          42       /* No mem for expandBuffer */
#define         E_NOMINAL_TIME     43
#define         E_DES_SIZE         44
#define         E_PACKET_SIZE      45
#define         E_END_REACHED      46
#define         E_TIME_STAMP       47
#define         E_SAMPLES          48
#define         E_MAX_DF_SIZE      49
#define         E_CHANNEL_AUTH     50
#define         E_UNCOMPRESS       51
#define         E_DATA_TYPE        52
#define         E_ACQUIRE          53
#define         E_INDEX            54
#define         S_OPTION_REQ       55
#define         S_SEND_OPTION_RES  56
#define         S_SEND_OPTION_REQ  57
#define         W_ACQUIRE          58
#define         S_ACQUIRE          59
#define         S_ACKNAK           60
#define         E_STR_SIZE         61
#define         E_AUTH_OFFSET      62
#define         E_OPTIONSW         63
#define         E_DATA_SIZE        64
#define         E_STATUS_SIZE      65
#define         E_NO_ORF           66
#define         S_DUPLICATE_DFF    67
#define         E_DATA             68
#define         E_DATA_FORMAT      69
#define         S_DISCONNECTED     70
#define         E_STEIM_COMPRESS   71
#define         E_ITYPE            72
#define         E_UNSUPPORTED      73
#define         E_SAVE_DATA        74
#define         S_SEND_ACKNAK      75
#define         E_NO_ORS           76
#define         E_2MANY_GAPS       77
#define         W_SEQUENCE         78
#define         W_DROP_GAP         79
#define         E_NO_CERT          80
#define         W_NEW_GAP          81
#define         E_OOPS             82
#define         E_NO_KEYS          83
#define         E_FRAME_AUTH       84
#define         E_STATION_NAME     85
#define         E_FRAME_TOO_SHORT  86
#define         E_SAMPLES_CAN      87
#define         E_NO_CHANNEL_SIG   88
#define         S_FILE_CHANGE      89
#define         S_COMRQ_RECEIVED   90
#define         S_COMMAND_RESPONSE 91
#define         E_COMMAND_SIZE     92
#define         E_CR_TIMEOUT       93
#define         E_CR_FAILED        94
#define         E_UNEXPECTED_TYPE  95
#define         S_PAF              96
#define         E_PAF              97
#define         E_NOT_AUTHORIZED   98
#define         S_SEND_CRF         99
#define         S_CONN_RES         100
#define         E_CONN_RES         101
#define         S_SEND_DATA        102
#define         S_SEND_DFF         103
#define         S_OPTION_RES       104
#define         S_COMRS_RECEIVED   105
#define         E_NO_CTRL_STATION  106
#define         S_COMRQ_SENT       107
#define         S_DFF_CREATED      108
#define         S_DF_CREATED       109
#define         W_ISSTOPPED        110
#define         E_SUB_TIME_STAMP   111
#define         E_CALIB_PER        112
#define         E_CALIB_FAC        113
#define         E_FRAME_TIME_LEN   114
#define         E_INTERNAL         115
#define         E_CHANNEL_ERRORS   116
#define         E_CHANNEL_WARNINGS 117
#define         E_CHANNEL_AUTH_ER  118
#define         E_CHANNEL_AUTH_WA  119
#define         W_COMPILED_WITHOUT 120
#define         E_CMD_TOO_OLD      121
#define         E_CMD_EXPECTED     122
#define         W_CALIB_INFO       123
#define         E_LOAD_RECORD      124
#define         E_INVALID_NAME     125
#define         E_CONFIG           126
#define         E_SAVE_RECORD      127
#define         E_SYNC_WFDISCS     128
#define         E_DESTINATION      129
#define         E_INVALID_GAP      130
#define         E_OUT_OF_SYNC      131
#define         E_CLOSED           132   /* E_CLOSED must be the last event */

/* ==== Defines and data structures for clf file ============================ */

#define         BAD_CRF          'C' /**< bad connection request frame */
#define         GOOD_CRF         'c' /**< good connection request frame */
#define         BAD_DATA         'D' /**< bad data frame */
#define         GOOD_DATA        'd' /**< good data frame */
#define         BAD_FORMAT       'F' /**< bad data format frame (only CD1) */
#define         GOOD_FORMAT      'f' /**< good data format frame (only CD1) */
#define         BAD_ALERT        'A' /**< bad alert frame */
#define         GOOD_ALERT       'a' /**< good alert frame */
#define         BAD_PAF          'P' /**< bad port assignment frame
                                          (only CD1) */
#define         GOOD_PAF         'p' /**< good port assignment frame
                                          (only CD1) */
#define         BAD_CON_RES      'E' /**< bad connection response frame */
#define         GOOD_CON_RES     'e' /**< good connection response
                                          (only CD1.1 */
#define         BAD_ACKNAK       'N' /**< bad Ack Nack (only CD1.1) */
#define         GOOD_ACKNAK      'n' /**< good Ack Nack (only CD1.1) */
#define         BAD_OPTION_REQ   'O' /**< bad option request (only CD1.1) */
#define         GOOD_OPTION_REQ  'o' /**< good option request (only CD1.1) */
#define         BAD_OPTION_RES   'S' /**< bad option response (only CD1.1) */
#define         GOOD_OPTION_RES  's' /**< good option response (only CD1.1) */
#define         VIRT_CONNECT     'k' /**< virtual connect only in clf file */
#define         VIRT_DISCONNECT  'l' /**< virtual disconnect only in clf file */
#define         VIRT_CHANNEL_NM  'h' /**< virtual channel name only
                                          in clf file */
#define         BAD_COMMAND_REQ  'M' /**< bad command request (only CD1.1) */
#define         GOOD_COMMAND_REQ 'm' /**< good command request (only CD1.1) */
#define         BAD_COMMAND_RES  'R' /**< bad command response (only CD1.1) */
#define         GOOD_COMMAND_RES 'r' /**< good command response (only CD1.1) */
#define         BAD_VIRT_DFF     'V' /**< bad virtual data format (CD1 only) */
#define         GOOD_VIRT_DFF    'v' /**< good virtual data format (CD1 only) */
#define         LAST_TYPE        'v'
#define         FRAME_UNKNOWN    '?' /**< unknown frame format */

#define IS_DFF(a)       (GOOD_FORMAT == (a) || BAD_FORMAT == (a) || \
                         GOOD_VIRT_DFF ==(a) || BAD_VIRT_DFF == (a))
#define IS_GOOD_FRAME(a) ((a) >= 'a' && (a) <= 'z')
#define MAKE_FRAME_BAD(a) ((a) -= ' ')

/* ==== Function prototypes ================================================= */

extern void logDFparsingEvents(station_t *pStation, uint32_t strictMode);
extern void logDFFparsingEvents(station_t *pStation, uint32_t strictMode);
extern void logDFauthenticationEvents(station_t *pStation, uint32_t strictMode);
extern void logCDevent(uint16_t whichEvent, station_t *station,
                       uint16_t reason, /*@null@*/ const portMap_t *portInfo,
                       uint32_t addInfo);
extern void logCDevent2(uint16_t eventType, uint16_t event,
                        CDevent_t *actEvent, uint32_t addInfo);
extern void updateStatistics(CDframeStatistics_t *pFrameStatistics,
                             const int bytes, const unsigned char isCR,
                             const char clfType);
extern /*@shared@*/ const char *getCDevent(uint16_t event);
extern /*@shared@*/ const char *getCDtype(unsigned char type);

#endif /* _EVENTS_H */
