
/*******************************************************/
/*  International Data Centre                          */
/*  Comprehensive Test Ban Treaty Organization         */
/*  Vienna                                             */
/*  Austria                                            */
/*******************************************************/

/**
 * @internal
 * @author    Johannes Schreitl
 *
 * $Log: init_sender.h,v $
 *
 * Revision 1.5  2015-03-31 GK
 * Add combined regular expression for sites and channels.
 *
 * Revision 1.4  2008-06-03 14:28:36  gerald
 * Add data consumer autocreate feature for applications which do not
 * need a data consumer (eg. cd2w, fileReaderAPI)
 *
 * Revision 1.3  2008/05/08 08:41:54  gerald
 * Fix typos
 *
 * Revision 1.2  2007/01/02 10:03:41  gerald
 * Improve fileReader API to work also without a configuration
 * file (for geotool)
 *
 * Revision 1.1  2006/03/28 08:58:54  gerald
 * Move functions used previously only by CDsender to libCD. These
 * functions are needed by the new Data Parsing Programming Interface
 *
 * Revision 1.10  2006/03/20 14:07:34  gerald
 * Add functions for creating a data consumer and a
 * station line structure
 *
 * Revision 1.9  2006/02/14 10:08:52  gerald
 * Add sender specific config structure and function for reading
 * sender specific config parameters from the config file
 *
 * Revision 1.8  2005/08/31 09:29:34  gerald
 * Make searchDCparam() function global available in src/cdsend
 *
 * Revision 1.7  2004/02/13 15:08:28  gerald
 * Fix splint warnings and add CD conform logging
 *
 * Revision 1.6  2003/11/20 13:11:10  gerald
 * Use station structure in CDsender and use stopProgram() instead of
 * stopSender().  Use more new functions from libCD.
 *
 * Revision 1.5  2003/11/20 08:29:51  karol
 * Remove watching thread, spawn of scheduler thread.
 *
 * Revision 1.4  2003/11/07 10:53:21  gerald
 * Remove own defines and use common defines now
 *
 * Revision 1.3  2003/05/12 12:32:38  johannes
 * multithreading for sending data added
 *
 * Revision 1.2  2003/04/03 11:06:09  johannes
 * minor fixes for Linux
 *
 * Revision 1.1  2003/04/02 11:09:15  johannes
 * first issue
 *
 */

#ifndef _INIT_SENDER_H
#define _INIT_SENDER_H

#include <arpa/inet.h>
#include "schedule.h"
#include "fileAPI.h"


/* ==== Data structures ===================================================== */

#define MIN_PORT_NUMBER 1024    /**< lowest accessible port */
#define PL_FIELDS_MIN  4     /**< allowed minimum number of fields in
                                  production line file */
#define PL_FIELDS_MAX  5     /**< allowed maximum number of fields in
                                  production line file */

#define AUTO_CREATE     1
#define NO_AUTO_CREATE  0

/* ==== Prototypes ========================================================== */
extern /*@null@*/
dcParams_t * createDC(const char *dcName, const char *IPandPort,
                      const char *dataConsumerFile, const char *appName);
extern /*@null@*/ prodLine_t *
createPL(const char *pLine, const char *productionLineFile,
         senderParams_t *pSenderParams, const char *appName, int dcAutoCreate);
extern void readDCfile(senderParams_t *pSenderParams, const char *appName);
extern void readPLfile(senderParams_t *pSenderParams, const char *appName,
                       int dcAutoCreate);
extern /*@dependent@*//*@null@*/
dcParams_t *searchDCparam( const char *pKey, /*@null@*/dcParams_t *pDcListHead);
extern void setSenderConfigExtension(configParams_t *pConfigData);
extern void readSenderConfigParams(const char *appName, const char *fileName,
                                   configParams_t *pConfigData);
extern void dumpSenderConfigurationParameter (configParams_t *pConfigData);
#endif /* _INIT_SENDER_H */
