#ifndef _INITTABLES_H
#define _INITTABLES_H

#include <netdb.h>
#include <stdio.h>
#include <string.h>

#ifdef HAVE_STDINT_H
#include <stdint.h>
#endif

/**
 * @internal
 * @author    Gerald Klinkl
 *
 * Revision 1.7  2014-11-10 GK
 * Add and update function references for doxygen
 *
 * Revision 1.6  2013-10-16 gerald
 * Fix Missing include on FreeBSD
 */

/* ==== Data structures ===================================================== */

/**
 * Used to scan for bit values in the config file
 *
 * If parameter 'name' is specified in the configuration file and has
 * the value 'value', set bit 'setValue' and reset 'resetValue'
 * in the systemConf variable
 */
typedef struct _bitParams
{
  const char *name;    /**< name of config parameter to search */
  const char *value;   /**< value of config parameter */
  uint32_t setValue;   /**< bit value to set if found */
  uint32_t resetValue; /**< bit value to reset if found */
} bitParams_t;

/**
 * Used to scan for number values in the config file
 *
 * If parameter 'name' is specified in the configuration file store
 * value in referenced variable
 */
typedef struct _nbrParams
{
  const char *name;        /**< name of config parameter to search */
  /*@shared@*/void *p2var;     /**< referenced variable to store value */
#define INT16U  1       /**< 16 bit unsigned integer */
#define INT32U  2       /**< 32 bit unsigned integer */
#define INT32S  3       /**< 32 bit signed integer */
  int iType;              /**< type of p2var (16/32 bit [un]signed) */
} nbrParams_t;

/**
 * Used to scan for string values in the config file
 *
 * If parameter 'name' is specified in the configuration file store
 * value in referenced string
 */
typedef struct _estrParams
{
  const char *name;        /**< name of config parameter to search */
  /*@shared@*/char *p2var; /**< string to store value in */
#define VALUE_REQUIRED 1   /**< optional value */
#define VALUE_OPTIONAL 2   /**< required value */
  int flag;                /**< type (optional or required) */
} strParams_t;

/**
 * Used to scan for string values in the config file
 *
 * If parameter 'name' is specified in the configuration file store
 * value in referenced string
 */
typedef struct _obsParams
{
  const char *pObsolete;    /**< name of obsolete config parameter to search */
  /*@null@*/char *pReplace; /**< name of config parameter replaces obsolete */
#define VALUE_CONTINUE 1   /**< continue parsing */
#define VALUE_STOP     2   /**< sop parsing with an error */
  int flag;                /**< type (optional or required) */
} obsParams_t;

/* ==== Prototypes ========================================================== */

extern int processBitParams(size_t tableSize, bitParams_t bitParams[],
                            uint32_t *value);
extern void dumpBitParams(size_t tableSize, bitParams_t bitParams[],
                          uint32_t value, int toStdout);
extern int processNumberParams(size_t tableSize, nbrParams_t nbrParams[]);
extern void dumpNumberParams(size_t tableSize, nbrParams_t nbrParams[]);
extern int processStringParams(size_t tableSize, strParams_t strParams[]);
extern void dumpStringParams(size_t tableSize, strParams_t strParams[]);
extern int processObsoleteParams(size_t tableSize, obsParams_t obsParams[]);

#endif /* _INITTABLES_H */
