/**
 * 2014-09-26			Ewald Peiszer, Zuhlke Engineering			Removed function declarations where function is defined in libstanoise
 *
 */



#ifndef LIBQCTOOL_H_
#define LIBQCTOOL_H

#include "qctool.h"
#include "libgparseP.h"
#include "libgODBC.h"
#include <time.h>
#include <unistd.h>

#define MAX_SUMM               (800)
#define NUMBER_OF_PERCENTILES  (20)

typedef struct
{
  double **psd_log;
  double **psd_log_std;
  double **freq;
  int    *samples;
  double *sample_rate;
  double  start_time;
  double  end_time;
  logical *good_sensor;
  char   *station_name;
  char  **sensor;
  char  **chan;
  int     nsensors;
  char   *psd_key;
  char   *shi_type;
  int     psd_id;
  char   *name;
  char   *dir;
  char   *out_dir;
  char   *dfile;
  double  lat;
  double  lon;
  int    *iterations;
  char   *window_type;
  double  enbw;
  double  main_window_length;
  double *sub_window_length;
  double  rov;
  int     *decon;
  int     endian;
  char   **decon_file;
  char   **recon_file;
  char   *local_day;
  char   *local_time;
  char   *units;
  double *calib;
  double *calper;
  char   *smoothing;
}PSD_STRUCT;

typedef struct
{
  char                   *sensor_name;
  char                   *chan;
  int                     f_bins;
  int                     p_bins;
  double                  *log_freq;
  double                  *log_psd;
  double                  *total_past;
  double                **psd_prob;
  logical                 good_sensor;
  int                     iterations;
  int                     decon;
  double                  sub_window_length;
}PSD_PROB_SENSOR_STRUCT;

typedef struct
{
  char                    *station_name;
  char                    *shi_type;
  char                    *window_type;
  double                   start_time;
  double                   end_time;
  double                   enbw;
  double                   main_window_length;
  int                      nsensors;
  int                      endian;
  double                   rov;
  PSD_PROB_SENSOR_STRUCT  *psd_prob_sensor;
}PSD_PROB_STRUCT;

typedef struct
{
  char  *in_dir;
  char  *sta;
  char  *job;
  char  *username;
  char  *passwd;
  char  *source;
  double start_time;
  int    number_of_intervals;
  double interval_length;
  double end_time;
  double main_window_length;
} SN_STATE_PARS;


typedef struct
{
  double *freq;
  double *dP;
  double *time;
  char   *chan;
  char   *sensor;
  int    *samples;
  int     j_sensor;
} SUMM_DAT;

typedef struct
{
  char *psd_file;
  int nsensors;
  int jdate;
  int hour;
  int min;
  int sec;
  double start_time;
  double end_time;
  SUMM_DAT *summ_dat;
}SUMM;

typedef struct
{
  char **sensor;
  char **chan;
} SC;

int         AddHourMinGroup(int root_grp,PSD_STRUCT *psd_struct);
int         BigOrLittleEndian();
logical     file_exists (char * filename);/* check file exists */
logical     ContainsString(char *string1,char *string2);
char       *CopyAfterEqual(char *target,  char *source);
/* copy after slash */
PSD_STRUCT *CopyPSDStruct(PSD_STRUCT *psd_struct_in);
PSD_STRUCT *CreatePSDStruct(int nsensors);
char       *EpochtimeToJdate(double epochtime);
void        FreePSDStruct(PSD_STRUCT *psd_struct);
void        FreeStation(STATION *station,int nchan);
void        FreeWavedata(WAVEDATA *wavedata);
char      **GenerateYearDay_list(double start_jdate,double end_jdate,int number_of_days);
/*get chanid - below*/
char      **GetChannels(char *chan_list,int *nchan);
complex   **GetDataBlock(double start_time,double end_time,WAVEDATA *wavedata);
WAVEDATA   *GetDotwAndWfdiscData(GeotoolStruct *gs,INPUT_PARAMETERS *ip, STATION *station,char *shi_type);
double      GetMaxSampleRate(PSD_STRUCT *psd_struct);
int         GetMaxSamples(PSD_STRUCT *psd_struct);
double     *GetNoDiffSampleRates(PSD_STRUCT *psd_struct);
char       *GetOutFileName(INPUT_PARAMETERS *ip,STATION *station);
void        GetSensorFromDataBlockTime(complex **a,complex *x,int i_sensor,int sample_points);
/*get sta lat lon*/
STATE_INFO *GetStateStaNoise(char *dir_filename,int *type_lookup,SN_STATE_PARS *sn_state_pars);
STATION    *GetStationInfo(GParseEnv gpe,INPUT_PARAMETERS *ip);
/*initialize net mod file*/
int         IsRecorded(STATION *station,char *sitechan_sta,char *sitechan_chan,int i_sensor);
double      JdateToEpoch(char *jdate_hour);
int         MkDir(char *dir);
/*net mod pdf*/
int         PlotProb(PSD_STRUCT *psd_struct,PSD_STRUCT *psd_struct_smooth, int *label, int *ordinal);
void        PsdProb(PSD_STRUCT *psd_struct);
PSD_STRUCT *ReadPSDData(char *in_name);
/*read psd netcdf4*/
PSD_PROB_STRUCT *ReadPsdProb(char *psd_name);
PSD_STRUCT *SanitizedPSDlog(PSD_STRUCT *psd_struct_in);
/*set state - below*/
char       *SHIStationType(char *station_name);
void        UpdateAveragePSD(PSD_STRUCT *psd_struct);
void        UpdateDayNightAveragePSD(PSD_STRUCT *psd_struct);
void        UpdateHighNoiseModel(PSD_STRUCT *psd_struct);
void        UpdateLowNoiseModel(PSD_STRUCT *psd_struct);
int         UpdateNetModFile(char *filename,char *pdf_filename, int root_grp,PSD_STRUCT *psd_struct_av);
/*update states - below*/
/*use day - below*/
logical     UseFile(char *filename,char *station_name, Vector vqcdata,double start_time,double end_time,int offset,char *idcx_account);
int         WriteNetModData(PSD_STRUCT *psd_struct_av,PSD_STRUCT *psd_struct);
void        WritePSDData(PSD_STRUCT *psd_struct);
int         WritePSD_NetCDF4(PSD_STRUCT *psd_struct);
/*write psd prob*/
/*write wfdisc data - below*/

#ifdef HAVE_LIBODBC
int        *SetState(SQLHDBC hdbc,double duration,double interval_length,double interval_start_time,
              int    number_of_intervals,
              int  *type_lookup,
              char  *sta,
              int   *state,
              int    nsensors,
              char **sensor,
              char **chan,
              char *idcx_account);
int         GetChanid(SQLHDBC hdbc, char *sta, char *chan);
int         UpdateState(SQLHDBC hdbc,STATE_INFO *state_info,double duration,double interval_length,char *sta,int *type_lookup,char *idcx_account);
logical     UseDay(SQLHDBC hdbc,char *filename,char *sta,Vector vqcdata,int *type_lookup,char *idcx_account);
int         WriteWfdiscData(SQLHDBC hdbc,char *filename,WAVEDATA *wavedata);
#endif


void        check_err(const int stat, const int line, const char *file); /*?*/



#endif
